=== modified file 'lib/canonical/launchpad/webapp/configure.zcml'
--- lib/canonical/launchpad/webapp/configure.zcml	2011-06-08 18:39:38 +0000
+++ lib/canonical/launchpad/webapp/configure.zcml	2011-07-01 15:36:23 +0000
@@ -440,6 +440,25 @@
         factory="canonical.launchpad.webapp.namespace.FormNamespaceView"
         />
 
+    <!-- Expose LaunchpadView methods. -->
+    <class class="canonical.launchpad.webapp.publisher.LaunchpadView">
+      <allow attributes="getCacheJson initialize" />
+    </class>
+
+    <!-- Create a namespace to render the model of any LaunchpadView-->
+    <view
+        name="model" type="*"
+        provides="zope.traversing.interfaces.ITraversable" for="*"
+        factory="canonical.launchpad.webapp.namespace.JsonModelNamespaceView"
+        permission="zope.Public"
+        />
+
+    <class class="canonical.launchpad.webapp.namespace.JsonModelNamespaceView">
+        <allow
+          attributes="__call__"
+          interface="zope.publisher.interfaces.browser.IBrowserPublisher" />
+    </class>
+
     <!-- Registrations to support +haproxy status url. -->
     <browser:page
         for="canonical.launchpad.webapp.interfaces.ILaunchpadRoot"

=== modified file 'lib/canonical/launchpad/webapp/namespace.py'
--- lib/canonical/launchpad/webapp/namespace.py	2011-05-27 21:03:22 +0000
+++ lib/canonical/launchpad/webapp/namespace.py	2011-07-28 13:58:14 +0000
@@ -1,11 +1,23 @@
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
+__metaclass__ = type
+
+__all__ = [
+    'FormNamespaceView',
+    'JsonModelNamespaceView',
+    ]
+
+
 from z3c.ptcompat import ViewPageTemplateFile
 from zope.app.pagetemplate.viewpagetemplatefile import BoundPageTemplate
+from zope.app.publisher.browser import getDefaultViewName
+from zope.component import getMultiAdapter
 from zope.security.proxy import removeSecurityProxy
 from zope.traversing.interfaces import TraversalError
 from zope.traversing.namespace import view
+from zope.interface import implements
+from zope.publisher.interfaces.browser import IBrowserPublisher
 
 from lp.app.browser.launchpadform import LaunchpadFormView
 
@@ -28,7 +40,7 @@
         context = removeSecurityProxy(self.context)
 
         if isinstance(context, LaunchpadFormView):
-            # Note: without explicitely creating the BoundPageTemplate here
+            # Note: without explicitly creating the BoundPageTemplate here
             # the view fails to render.
             context.index = BoundPageTemplate(FormNamespaceView.template,
                                               context)
@@ -36,3 +48,42 @@
             raise TraversalError("The URL does not correspond to a form.")
 
         return self.context
+
+
+class JsonModelNamespaceView(view):
+    """A namespace view to handle traversals with ++model++.
+
+    Use of this namespace is only guaranteed to work if it is fully populated
+    by a view's `initialize` method.  Any objects added after the call to
+    initialize will not be presented by the namespace.
+    """
+
+    implements(IBrowserPublisher)
+
+    def traverse(self, name, ignored):
+        """Model traversal adapter.
+
+        This adapter allows any LaunchpadView to render its JSON cache.
+        """
+        return self
+
+    def browserDefault(self, request):
+        # Tell traversal to stop, already.
+        return self, None
+
+    def __call__(self):
+        """Return the JSON cache."""
+        if IBrowserPublisher.providedBy(self.context):
+            view = self.context
+        else:
+            defaultviewname = getDefaultViewName(
+                self.context, self.request)
+            view = getMultiAdapter(
+                (self.context, self.request), name=defaultviewname)
+        if view is None:
+            return
+        naked_view = removeSecurityProxy(view)
+        naked_view.initialize()
+        cache = naked_view.getCacheJSON()
+        self.request.response.setHeader('content-type', 'application/json')
+        return cache

=== modified file 'lib/canonical/launchpad/webapp/publisher.py'
--- lib/canonical/launchpad/webapp/publisher.py	2011-07-21 22:45:05 +0000
+++ lib/canonical/launchpad/webapp/publisher.py	2011-07-27 13:23:38 +0000
@@ -26,6 +26,7 @@
     ]
 
 import httplib
+import simplejson
 
 from zope.app import zapi
 from zope.app.publisher.interfaces.xmlrpc import IXMLRPCView
@@ -51,7 +52,14 @@
     )
 from zope.traversing.browser.interfaces import IAbsoluteURL
 
+from lazr.restful import (
+    EntryResource,
+    ResourceJSONEncoder,
+    )
 from lazr.restful.declarations import error_status
+from lazr.restful.interfaces import IJSONRequestCache
+
+from lazr.restful.tales import WebLayerAPI
 
 from canonical.launchpad.layers import (
     LaunchpadLayer,
@@ -340,6 +348,17 @@
 
     info_message = property(_getInfoMessage, _setInfoMessage)
 
+    def getCacheJSON(self):
+        if self.user is not None:
+            cache = dict(IJSONRequestCache(self.request).objects)
+        else:
+            cache = dict()
+        if WebLayerAPI(self.context).is_entry:
+            cache['context'] = self.context
+        return simplejson.dumps(
+            cache, cls=ResourceJSONEncoder,
+            media_type=EntryResource.JSON_TYPE)
+
 
 class LaunchpadXMLRPCView(UserAttributeCache):
     """Base class for writing XMLRPC view code."""

=== added file 'lib/canonical/launchpad/webapp/templates/launchpad-model.pt'
--- lib/canonical/launchpad/webapp/templates/launchpad-model.pt	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/webapp/templates/launchpad-model.pt	2011-07-27 15:24:03 +0000
@@ -0,0 +1,13 @@
+<div
+  xmlns="http://www.w3.org/1999/xhtml"
+  xmlns:tal="http://xml.zope.org/namespaces/tal"
+  xmlns:metal="http://xml.zope.org/namespaces/metal"
+  xmlns:i18n="http://xml.zope.org/namespaces/i18n"
+  xml:lang="en"
+  lang="en"
+  dir="ltr"
+  i18n:domain="launchpad">
+
+  <tal:json replace="structure context/getCacheJSON" />
+
+</div>

=== modified file 'lib/canonical/launchpad/webapp/tests/test_publisher.py'
--- lib/canonical/launchpad/webapp/tests/test_publisher.py	2010-10-12 01:11:41 +0000
+++ lib/canonical/launchpad/webapp/tests/test_publisher.py	2011-07-18 19:27:35 +0000
@@ -5,10 +5,96 @@
     DocTestSuite,
     ELLIPSIS,
     )
+from unittest import TestLoader, TestSuite
+
+from lazr.restful.interfaces import IJSONRequestCache
+import simplejson
+from zope.component import getUtility
+
+from canonical.testing.layers import DatabaseFunctionalLayer
+from canonical.launchpad.webapp.publisher import LaunchpadView
+from canonical.launchpad.webapp.servers import LaunchpadTestRequest
+from lp.services.worlddata.interfaces.country import ICountrySet
+from lp.testing import (
+    logout,
+    person_logged_in,
+    TestCaseWithFactory,
+    )
 
 from canonical.launchpad.webapp import publisher
 
 
+class TestLaunchpadView(TestCaseWithFactory):
+
+    layer = DatabaseFunctionalLayer
+
+    def test_getCacheJSON_non_resource_context(self):
+        view = LaunchpadView(object(), LaunchpadTestRequest())
+        self.assertEqual('{}', view.getCacheJSON())
+
+    @staticmethod
+    def getCanada():
+        return getUtility(ICountrySet)['CA']
+
+    def assertIsCanada(self, json_dict):
+        self.assertIs(None, json_dict['description'])
+        self.assertEqual('CA', json_dict['iso3166code2'])
+        self.assertEqual('CAN', json_dict['iso3166code3'])
+        self.assertEqual('Canada', json_dict['name'])
+        self.assertIs(None, json_dict['title'])
+        self.assertContentEqual(
+            ['description', 'http_etag', 'iso3166code2', 'iso3166code3',
+             'name', 'resource_type_link', 'self_link', 'title'],
+            json_dict.keys())
+
+    def test_getCacheJSON_resource_context(self):
+        view = LaunchpadView(self.getCanada(), LaunchpadTestRequest())
+        json_dict = simplejson.loads(view.getCacheJSON())['context']
+        self.assertIsCanada(json_dict)
+
+    def test_getCacheJSON_non_resource_object(self):
+        request = LaunchpadTestRequest()
+        view = LaunchpadView(object(), request)
+        IJSONRequestCache(request).objects['my_bool'] = True
+        with person_logged_in(self.factory.makePerson()):
+            self.assertEqual('{"my_bool": true}', view.getCacheJSON())
+
+    def test_getCacheJSON_resource_object(self):
+        request = LaunchpadTestRequest()
+        view = LaunchpadView(object(), request)
+        IJSONRequestCache(request).objects['country'] = self.getCanada()
+        with person_logged_in(self.factory.makePerson()):
+            json_dict = simplejson.loads(view.getCacheJSON())['country']
+        self.assertIsCanada(json_dict)
+
+    def test_getCacheJSON_context_overrides_objects(self):
+        request = LaunchpadTestRequest()
+        view = LaunchpadView(self.getCanada(), request)
+        IJSONRequestCache(request).objects['context'] = True
+        with person_logged_in(self.factory.makePerson()):
+            json_dict = simplejson.loads(view.getCacheJSON())['context']
+        self.assertIsCanada(json_dict)
+
+    def test_getCache_anonymous(self):
+        request = LaunchpadTestRequest()
+        view = LaunchpadView(self.getCanada(), request)
+        self.assertIs(None, view.user)
+        IJSONRequestCache(request).objects['my_bool'] = True
+        json_dict = simplejson.loads(view.getCacheJSON())
+        self.assertIsCanada(json_dict['context'])
+        self.assertFalse('my_bool' in json_dict)
+
+    def test_getCache_anonymous_obfuscated(self):
+        request = LaunchpadTestRequest()
+        branch = self.factory.makeBranch(name='user@domain')
+        logout()
+        view = LaunchpadView(branch, request)
+        self.assertIs(None, view.user)
+        self.assertNotIn('user@domain', view.getCacheJSON())
+
+
 def test_suite():
-    suite = DocTestSuite(publisher, optionflags=ELLIPSIS)
+    suite = TestSuite()
+    suite.addTest(DocTestSuite(publisher, optionflags=ELLIPSIS))
+    suite.addTest(TestLoader().loadTestsFromName(__name__))
     return suite

=== added file 'lib/canonical/launchpad/webapp/tests/test_view_model.py'
--- lib/canonical/launchpad/webapp/tests/test_view_model.py	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/webapp/tests/test_view_model.py	2011-07-27 14:01:02 +0000
@@ -0,0 +1,167 @@
+# Copyright 2011 Canonical Ltd.  All rights reserved.
+
+"""Tests for the user requested oops using ++oops++ traversal."""
+
+__metaclass__ = type
+
+
+from lazr.restful.interfaces import IJSONRequestCache
+from lazr.restful.utils import get_current_browser_request
+from simplejson import loads
+from testtools.matchers import KeysEqual
+from zope.configuration import xmlconfig
+
+from canonical.launchpad.webapp import LaunchpadView
+from canonical.launchpad.webapp.publisher import canonical_url
+from canonical.launchpad.webapp.namespace import JsonModelNamespaceView
+import canonical.launchpad.webapp.tests
+from canonical.testing.layers import DatabaseFunctionalLayer
+from lp.app.browser.launchpadform import LaunchpadFormView
+from lp.testing import (
+    ANONYMOUS,
+    BrowserTestCase,
+    login,
+    logout,
+    TestCaseWithFactory,
+    )
+
+
+class FakeView:
+    """A view object that just has a fake context and request."""
+    def __init__(self):
+        self.context = object()
+        self.request = object()
+
+
+class TestJsonModelNamespace(TestCaseWithFactory):
+    """Test that traversal to ++model++ returns a namespace."""
+    layer = DatabaseFunctionalLayer
+
+    def setUp(self):
+        TestCaseWithFactory.setUp(self)
+        login(ANONYMOUS)
+
+    def tearDown(self):
+        logout()
+        TestCaseWithFactory.tearDown(self)
+
+    def test_JsonModelNamespace_traverse_non_LPview(self):
+        # Test traversal for JSON model namespace,
+        # ++model++ for a non-LaunchpadView context.
+        request = get_current_browser_request()
+        context = object()
+        view = FakeView()
+        namespace = JsonModelNamespaceView(context, request)
+        result = namespace.traverse(view, None)
+        self.assertEqual(result, namespace)
+
+    def test_JsonModelNamespace_traverse_LPView(self):
+        # Test traversal for JSON model namespace,
+        # ++model++ for a non-LaunchpadView context.
+        request = get_current_browser_request()
+        context = object()
+        view = LaunchpadView(context, request)
+        namespace = JsonModelNamespaceView(view, request)
+        result = namespace.traverse(view, None)
+        self.assertEqual(result, namespace)
+
+    def test_JsonModelNamespace_traverse_LPFormView(self):
+        # Test traversal for JSON model namespace,
+        # ++model++ for a non-LaunchpadView context.
+        request = get_current_browser_request()
+        context = object()
+        view = LaunchpadFormView(context, request)
+        namespace = JsonModelNamespaceView(view, request)
+        result = namespace.traverse(view, None)
+        self.assertEqual(result, namespace)
+
+
+class BaseProductModelTestView(LaunchpadView):
+    def initialize(self):
+        # Ensure initialize does not put anything in the cache.
+        pass
+
+
+class TestJsonModelView(BrowserTestCase):
+
+    layer = DatabaseFunctionalLayer
+
+    def setUp(self):
+        TestCaseWithFactory.setUp(self)
+        login(ANONYMOUS)
+        self.product = self.factory.makeProduct(name="test-product")
+        self.url = canonical_url(self.product) + '/+modeltest/++model++'
+
+    def tearDown(self):
+        logout()
+        TestCaseWithFactory.tearDown(self)
+
+    def configZCML(self):
+        # Register the ZCML for our test view.  Note the view class must be
+        # registered first.
+        xmlconfig.string("""
+          <configure
+              xmlns:browser="http://namespaces.zope.org/browser">
+              <include package="canonical.launchpad.webapp"
+                  file="meta.zcml" />
+              <include package="zope.app.zcmlfiles" file="meta.zcml" />
+              <browser:page
+                name="+modeltest"
+                for="lp.registry.interfaces.product.IProduct"
+                class="canonical.launchpad.webapp.tests.ProductModelTestView"
+                permission="zope.Public"
+                />
+          </configure>""")
+
+    def test_JsonModel_default_cache(self):
+        # If nothing is added to the class by the view, the cache will only
+        # have the context.
+        class ProductModelTestView(BaseProductModelTestView):
+            pass
+        canonical.launchpad.webapp.tests.ProductModelTestView = \
+            ProductModelTestView
+        self.configZCML()
+        browser = self.getUserBrowser(self.url)
+        cache = loads(browser.contents)
+        self.assertEqual(['context'], cache.keys())
+
+    def test_JsonModel_custom_cache(self):
+        # Adding an item to the cache in the initialize method results in it
+        # being in the cache.
+        class ProductModelTestView(BaseProductModelTestView):
+            def initialize(self):
+                request = get_current_browser_request()
+                target_info = {}
+                target_info['title'] = "The Title"
+                cache = IJSONRequestCache(request).objects
+                cache['target_info'] = target_info
+        canonical.launchpad.webapp.tests.ProductModelTestView = \
+            ProductModelTestView
+        self.configZCML()
+        browser = self.getUserBrowser(self.url)
+        cache = loads(browser.contents)
+        self.assertThat(cache, KeysEqual('context', 'target_info'))
+
+    def test_JsonModel_custom_cache_wrong_method(self):
+        # Adding an item to the cache in some other method is not recognized,
+        # even if it called as part of normal rendering.
+        class ProductModelTestView(BaseProductModelTestView):
+            def initialize(self):
+                request = get_current_browser_request()
+                target_info = {}
+                target_info['title'] = "The Title"
+                cache = IJSONRequestCache(request).objects
+                cache['target_info'] = target_info
+
+            def render(self):
+                request = get_current_browser_request()
+                other_info = {}
+                other_info['spaz'] = "Stuff"
+                IJSONRequestCache(request).objects['other_info'] = other_info
+
+        canonical.launchpad.webapp.tests.ProductModelTestView = \
+            ProductModelTestView
+        self.configZCML()
+        browser = self.getUserBrowser(self.url)
+        cache = loads(browser.contents)
+        self.assertThat(cache, KeysEqual('context', 'target_info'))

=== modified file 'lib/lp/app/javascript/client.js'
--- lib/lp/app/javascript/client.js	2011-04-26 20:33:18 +0000
+++ lib/lp/app/javascript/client.js	2011-07-15 18:03:04 +0000
@@ -164,6 +164,50 @@
     }
 };
 
+
+/**
+ * Get the URL of the view for an Entry
+ * @method get_view_url
+ * @param {Entry} entry
+ * @param {String} view_name
+ * @param {String} namespace
+ * @return {String} URL
+ */
+module.get_view_url = function(entry, view_name, namespace){
+    entry_url = Y.lp.get_url_path(entry.get('web_link'));
+    return entry_url + '/' + view_name + '/++' + namespace + '++';
+};
+
+
+/**
+ * Get the URL of the form for a view for an Entry
+ * @method get_form_url
+ * @param {Entry} entry
+ * @param {String} view_name
+ * @return {String} URL
+ */
+module.get_form_url = function(entry, view_name) {
+    return module.get_view_url(entry, view_name, 'form');
+};
+
+
+module.load_model = function(entry, view_name, config){
+    var url = module.get_view_url(entry, view_name, 'model');
+    var old_on_success = config.on.success;
+    var on = config.on;
+    on.success = module.wrap_resource_on_success;
+    var y_config = {
+        on: on,
+        'arguments': [entry.lp_client, url, old_on_success, false]
+    };
+    var io = config.io;
+    if (!Y.Lang.isValue(io)) {
+        io = Y.io;
+    }
+    io(url, y_config);
+};
+
+
 module.add_accept = function(config, headers) {
     if (headers === undefined) {
         headers = {};
@@ -702,6 +746,8 @@
     },
 
     'wrap_resource': function(uri, representation) {
+        var key;
+        var new_representation;
         /* Given a representation, turn it into a subclass of Resource. */
         if (representation === null || representation === undefined) {
             return representation;
@@ -717,6 +763,25 @@
                 // It's a list. Treat it as a collection;
                 // it should be slicable.
                 return new Collection(this, representation, uri);
+            } else if (Y.Lang.isObject(representation)) {
+                // It's an Array or mapping.  Recurse into it.
+                if (Y.Lang.isArray(representation)) {
+                    new_representation = [];
+                }
+                else {
+                    new_representation = {};
+                }
+                for (key in representation) {
+                    if (representation.hasOwnProperty(key)) {
+                        var value = representation[key];
+                        if (Y.Lang.isValue(value)) {
+                            value = this.wrap_resource(
+                                value.self_link, value);
+                        }
+                        new_representation[key] = value;
+                    }
+                }
+                return new_representation;
             } else {
                 // It's a random JSON object. Leave it alone.
                 return representation;
@@ -844,7 +909,8 @@
 module.FormErrorHandler = FormErrorHandler;
 
 
-}, "0.1" ,{"requires":["attribute", "io", "json-parse", "json-stringify"]});
+}, "0.1",
+    {"requires":["attribute", "io", "json-parse", "json-stringify", "lp"]});
 
 
 YUI.add('lp.client.plugins', function (Y) {

=== added file 'lib/lp/app/javascript/testing/iorecorder.js'
--- lib/lp/app/javascript/testing/iorecorder.js	1970-01-01 00:00:00 +0000
+++ lib/lp/app/javascript/testing/iorecorder.js	2011-07-27 18:10:47 +0000
@@ -0,0 +1,47 @@
+/* Copyright (c) 2011, Canonical Ltd. All rights reserved. */
+
+YUI.add('lp.testing.iorecorder', function(Y) {
+    namespace = Y.namespace("lp.testing.iorecorder");
+    function MockHttpResponse () {
+        this.responseText = '[]';
+        this.responseHeaders = {};
+    }
+
+    MockHttpResponse.prototype = {
+        setResponseHeader: function (header, value) {
+            this.responseHeaders[header] = value;
+        },
+
+        getResponseHeader: function(header) {
+            return this.responseHeaders[header];
+        }
+    };
+    namespace.MockHttpResponse = MockHttpResponse;
+
+
+    function IORecorderRequest(url, config){
+        this.url = url;
+        this.config = config;
+        this.response = null;
+    }
+
+
+    IORecorderRequest.prototype.success = function(value, headers){
+        this.response = new MockHttpResponse();
+        this.response.setResponseHeader(
+            'Content-Type', headers['Content-Type']);
+        this.response.responseText = value;
+        this.config.on.success(null, this.response, this.config['arguments']);
+    };
+
+
+    function IORecorder(){
+        this.requests = [];
+    }
+
+
+    IORecorder.prototype.do_io = function(url, config){
+        this.requests.push(new IORecorderRequest(url, config));
+    };
+    namespace.IORecorder = IORecorder;
+}, '0.1', {});

=== modified file 'lib/lp/app/javascript/tests/test_lp_client.html'
--- lib/lp/app/javascript/tests/test_lp_client.html	2011-07-08 06:06:15 +0000
+++ lib/lp/app/javascript/tests/test_lp_client.html	2011-07-15 20:01:48 +0000
@@ -1,7 +1,7 @@
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
 <html>
   <head>
-  <title>Launchpad lp.client moduel</title>
+  <title>Launchpad lp.client module</title>
 
   <!-- YUI and test setup -->
   <script type="text/javascript"
@@ -10,8 +10,11 @@
   <link rel="stylesheet" href="../../../app/javascript/testing/test.css" />
   <script type="text/javascript"
           src="../../../app/javascript/testing/testrunner.js"></script>
+  <script type="text/javascript"
+          src="../../../app/javascript/testing/iorecorder.js"></script>
 
   <!-- The module under test -->
+  <script type="text/javascript" src="../lp.js"></script>
   <script type="text/javascript" src="../client.js"></script>
 
   <!-- The test suite -->

=== modified file 'lib/lp/app/javascript/tests/test_lp_client.js'
--- lib/lp/app/javascript/tests/test_lp_client.js	2011-07-08 05:12:39 +0000
+++ lib/lp/app/javascript/tests/test_lp_client.js	2011-07-28 15:25:04 +0000
@@ -1,7 +1,7 @@
 /* Copyright (c) 2011, Canonical Ltd. All rights reserved. */
 
-YUI().use('lp.testing.runner', 'test', 'console', 'lp.client',
-          'escape', function(Y) {
+YUI().use('lp.testing.runner', 'lp.testing.iorecorder', 'test', 'console',
+          'lp.client', 'escape', function(Y) {
 
 var Assert = Y.Assert;  // For easy access to isTrue(), etc.
 
@@ -58,6 +58,86 @@
       Assert.areEqual(
           get_field_uri("/has/slash/", "field"),
           "/api/devel/has/slash/field");
+    },
+    test_view_url: function() {
+        entry_repr = {web_link: 'http://example.com/context'};
+        var context = new Y.lp.client.Entry(null, entry_repr, null);
+        expected = '/context/+myview/++mynamespace++';
+        actual = Y.lp.client.get_view_url(context, '+myview', 'mynamespace');
+        Assert.areEqual(expected, actual);
+    },
+    test_get_form_url: function() {
+        entry_repr = {web_link: 'http://example.com/context'};
+        var context = new Y.lp.client.Entry(null, entry_repr, null);
+        expected = '/context/+myview/++form++';
+        actual = Y.lp.client.get_form_url(context, '+myview');
+        Assert.areEqual(expected, actual);
+    },
+    test_load_model: function(){
+        var recorder = new Y.lp.testing.iorecorder.IORecorder();
+        Assert.areEqual(0, recorder.requests.length);
+        var mylist = [];
+        var config = {
+            io: Y.bind(recorder.do_io, recorder),
+            on: {
+                success: Y.bind(mylist.push, mylist)
+            }
+        };
+        var entry_repr = {web_link: 'http://example.com/context'};
+        var client = new Y.lp.client.Launchpad();
+        var context = new Y.lp.client.Entry(client, entry_repr, null);
+        Y.lp.client.load_model(context, '+myview', config);
+        var request = recorder.requests[0];
+        Assert.areEqual('/context/+myview/++model++', request.url);
+        request.success(
+            '{"boolean": true, "entry": {"resource_type_link": "foo"}}',
+            {'Content-Type': 'application/json'});
+        var result = mylist[0];
+        Assert.areSame(true, result.boolean);
+        Assert.isInstanceOf(Y.lp.client.Entry, result.entry);
+        Assert.areSame('foo', result.entry.get('resource_type_link'));
+    },
+    test_wrap_resource_nested_mapping: function() {
+        // wrap_resource produces mappings of plain object literals. These can
+        // be nested and have Entries in them.
+        var foo = {
+            baa: {},
+            bar: {
+                baz: {
+                    resource_type_link: 'qux'}
+            }
+        };
+        foo = new Y.lp.client.Launchpad().wrap_resource(null, foo);
+        Y.Assert.isInstanceOf(Y.lp.client.Entry, foo.bar.baz);
+    },
+    test_wrap_resource_nested_array: function() {
+        // wrap_resource produces arrays of array literals. These can
+        // be nested and have Entries in them.
+        var foo = [[{resource_type_link: 'qux'}]];
+        foo = new Y.lp.client.Launchpad().wrap_resource(null, foo);
+        Y.Assert.isInstanceOf(Y.lp.client.Entry, foo[0][0]);
+    },
+    test_wrap_resource_creates_array: function() {
+        // wrap_resource creates new arrays, rather than reusing the existing
+        // one.
+        var foo = ['a'];
+        var bar = new Y.lp.client.Launchpad().wrap_resource(null, foo);
+        Y.Assert.areNotSame(foo, bar);
+    },
+    test_wrap_resource_creates_mapping: function() {
+        // wrap_resource creates new mappings, rather than reusing the existing
+        // one.
+        var foo = {a: 'b'};
+        var bar = new Y.lp.client.Launchpad().wrap_resource(null, foo);
+        Y.Assert.areNotSame(foo, bar);
+    },
+    test_wrap_resource_null: function() {
+        // wrap_resource handles null correctly.
+        var foo = {
+            bar: null
+        };
+        foo = new Y.lp.client.Launchpad().wrap_resource(null, foo);
+        Y.Assert.isNull(foo.bar);
     }
 }));
 
@@ -208,28 +288,13 @@
       }
 }));
 
-function MockHttpResponse () {
-    this.responseText = '[]';
-    this.responseHeaders = {};
-}
-
-MockHttpResponse.prototype = {
-    setResponseHeader: function (header, value) {
-        this.responseHeaders[header] = value;
-    },
-
-    getResponseHeader: function(header) {
-        return this.responseHeaders[header];
-    }
-};
-
 suite.add(new Y.Test.Case({
     name: "lp.client.notifications",
 
     setUp: function() {
         this.client = new Y.lp.client.Launchpad();
         this.args=[this.client, null, this._on_success, false];
-        this.response = new MockHttpResponse();
+        this.response = new Y.lp.testing.iorecorder.MockHttpResponse();
         this.response.setResponseHeader('Content-Type', 'application/json');
     },
 

=== modified file 'lib/lp/app/templates/base-layout-macros.pt'
--- lib/lp/app/templates/base-layout-macros.pt	2011-07-08 11:46:41 +0000
+++ lib/lp/app/templates/base-layout-macros.pt	2011-07-15 14:10:46 +0000
@@ -168,15 +168,9 @@
       tal:content="string:LP.links['${key}'] =
                    '${links/?key/fmt:api_url}';">
     </script>
-    <script tal:repeat="key objects"
-      tal:content="string:LP.cache['${key}'] =
-                   ${objects/?key/webservice:json};">
-    </script>
   </tal:cache>
 
-  <script tal:condition="context/webservice:is_entry"
-    tal:content="string:LP.cache['context'] =
-                 ${context/webservice:json/fmt:obfuscate-email};">
+  <script tal:content="string:LP.cache = ${view/getCacheJSON};">
   </script>
 </metal:lp-client-cache>
 

=== modified file 'lib/lp/app/webservice/marshallers.py'
--- lib/lp/app/webservice/marshallers.py	2011-07-12 10:02:51 +0000
+++ lib/lp/app/webservice/marshallers.py	2011-07-27 17:34:09 +0000
@@ -13,9 +13,10 @@
 from lazr.restful.marshallers import (
     TextFieldMarshaller as LazrTextFieldMarshaller,
     )
-from zope.app.security.interfaces import IUnauthenticatedPrincipal
+from zope.component import getUtility
 
 from lp.services.utils import obfuscate_email
+from canonical.launchpad.webapp.interfaces import ILaunchBag
 
 
 class TextFieldMarshaller(LazrTextFieldMarshaller):
@@ -26,7 +27,7 @@
 
         Return the value as is.
         """
-        if (value is not None and
-                IUnauthenticatedPrincipal.providedBy(self.request.principal)):
+
+        if (value is not None and getUtility(ILaunchBag).user is None):
             return obfuscate_email(value)
         return value

=== modified file 'lib/lp/app/webservice/tests/test_marshallers.py'
--- lib/lp/app/webservice/tests/test_marshallers.py	2011-07-12 10:02:51 +0000
+++ lib/lp/app/webservice/tests/test_marshallers.py	2011-07-27 17:34:09 +0000
@@ -6,17 +6,19 @@
 __metaclass__ = type
 
 import transaction
-from zope.component import getUtility
 
 from canonical.launchpad.testing.pages import (
     LaunchpadWebServiceCaller,
     webservice_for_person,
     )
-from canonical.launchpad.webapp.interfaces import IPlacelessAuthUtility
 from canonical.launchpad.webapp.servers import WebServiceTestRequest
 from canonical.testing.layers import DatabaseFunctionalLayer
 from lp.app.webservice.marshallers import TextFieldMarshaller
-from lp.testing import logout, TestCaseWithFactory
+from lp.testing import (
+    logout,
+    person_logged_in,
+    TestCaseWithFactory,
+    )
 
 
 def ws_url(bug):
@@ -28,28 +30,17 @@
 
     layer = DatabaseFunctionalLayer
 
-    def _makeRequest(self, is_anonymous):
-        """Create either an anonymous or authenticated request."""
-        request = WebServiceTestRequest()
-        if is_anonymous:
-            request.setPrincipal(
-                getUtility(IPlacelessAuthUtility).unauthenticatedPrincipal())
-        else:
-            request.setPrincipal(self.factory.makePerson())
-        return request
-
     def test_unmarshall_obfuscated(self):
-        # Data is obfuccated if the request is anonynous.
-        request = self._makeRequest(is_anonymous=True)
-        marshaller = TextFieldMarshaller(None, request)
+        # Data is obfuscated if the user is anonynous.
+        marshaller = TextFieldMarshaller(None, WebServiceTestRequest())
         result = marshaller.unmarshall(None, u"foo@example.com")
         self.assertEqual(u"<email address hidden>", result)
 
     def test_unmarshall_not_obfuscated(self):
-        # Data is not obfuccated if the request is authenticated.
-        request = self._makeRequest(is_anonymous=False)
-        marshaller = TextFieldMarshaller(None, request)
-        result = marshaller.unmarshall(None, u"foo@example.com")
+        # Data is not obfuccated if the user is authenticated.
+        marshaller = TextFieldMarshaller(None, WebServiceTestRequest())
+        with person_logged_in(self.factory.makePerson()):
+            result = marshaller.unmarshall(None, u"foo@example.com")
         self.assertEqual(u"foo@example.com", result)
 
 

=== modified file 'lib/lp/blueprints/browser/configure.zcml'
--- lib/lp/blueprints/browser/configure.zcml	2011-06-19 13:32:15 +0000
+++ lib/lp/blueprints/browser/configure.zcml	2011-07-01 11:27:27 +0000
@@ -135,6 +135,7 @@
         permission="zope.Public"/>
     <browser:pages
         for="lp.blueprints.interfaces.sprint.IHasSprints"
+        class="canonical.launchpad.webapp.LaunchpadView"
         facet="overview"
         permission="zope.Public">
         <browser:page

=== modified file 'lib/lp/bugs/browser/bugtarget.py'
--- lib/lp/bugs/browser/bugtarget.py	2011-06-21 08:36:42 +0000
+++ lib/lp/bugs/browser/bugtarget.py	2011-07-28 17:31:16 +0000
@@ -1202,28 +1202,28 @@
 
     @property
     def series_list(self):
-        if IDistribution(self.context, None):
-            series = self.context.series
-        elif IProduct(self.context, None):
-            series = self.context.series
-        elif IDistroSeries(self.context, None):
+        if IDistroSeries(self.context, None):
             series = self.context.distribution.series
+        elif IDistribution(self.context, None):
+            series = self.context.series
         elif IProductSeries(self.context, None):
             series = self.context.product.series
+        elif IProduct(self.context, None):
+            series = self.context.series
         else:
             raise AssertionError("series_list called with illegal context")
         return list(series)
 
     @property
     def milestones_list(self):
-        if IDistribution(self.context, None):
-            milestone_resultset = self.context.milestones
-        elif IProduct(self.context, None):
-            milestone_resultset = self.context.milestones
-        elif IDistroSeries(self.context, None):
+        if IDistroSeries(self.context, None):
             milestone_resultset = self.context.distribution.milestones
+        elif IDistribution(self.context, None):
+            milestone_resultset = self.context.milestones
         elif IProductSeries(self.context, None):
             milestone_resultset = self.context.product.milestones
+        elif IProduct(self.context, None):
+            milestone_resultset = self.context.milestones
         else:
             raise AssertionError("milestones_list called with illegal context")
         return list(milestone_resultset)
@@ -1245,14 +1245,14 @@
         if not series_list:
             return series_buglistings
         # This would be better as delegation not a case statement.
-        if IDistribution(self.context, None):
-            backlink = BugSummary.distroseries_id
+        if IDistroSeries(self.context, None):
+            backlink = BugSummary.distroseries_id
+        elif IDistribution(self.context, None):
+            backlink = BugSummary.distroseries_id
+        elif IProductSeries(self.context, None):
+            backlink = BugSummary.productseries_id
         elif IProduct(self.context, None):
             backlink = BugSummary.productseries_id
-        elif IDistroSeries(self.context, None):
-            backlink = BugSummary.distroseries_id
-        elif IProductSeries(self.context, None):
-            backlink = BugSummary.productseries_id
         else:
             raise AssertionError("illegal context %r" % self.context)
         counts = bug_task_set.countBugs(self.user, series_list, (backlink,))

=== modified file 'lib/lp/bugs/browser/configure.zcml'
--- lib/lp/bugs/browser/configure.zcml	2011-07-21 05:13:08 +0000
+++ lib/lp/bugs/browser/configure.zcml	2011-07-27 13:23:38 +0000
@@ -852,6 +852,7 @@
             name="+index"/>
         <browser:pages
             for="lp.bugs.interfaces.bugtracker.IRemoteBug"
+            class="canonical.launchpad.webapp.LaunchpadView"
             permission="zope.Public">
             <browser:page
                 name="+index"

=== modified file 'lib/lp/bugs/browser/cve.py'
--- lib/lp/bugs/browser/cve.py	2011-02-24 15:30:54 +0000
+++ lib/lp/bugs/browser/cve.py	2011-07-01 11:27:27 +0000
@@ -20,6 +20,7 @@
     canonical_url,
     ContextMenu,
     GetitemNavigation,
+    LaunchpadView,
     Link,
     )
 from canonical.launchpad.webapp.batching import BatchNavigator
@@ -122,11 +123,10 @@
     heading = 'Remove links to bug reports'
 
 
-class CveSetView:
+class CveSetView(LaunchpadView):
 
     def __init__(self, context, request):
-        self.context = context
-        self.request = request
+        super(CveSetView, self).__init__(context, request)
         self.notices = []
         self.results = None
         self.text = self.request.form.get('text', None)

=== modified file 'lib/lp/code/browser/bazaar.py'
--- lib/lp/code/browser/bazaar.py	2010-08-31 11:24:54 +0000
+++ lib/lp/code/browser/bazaar.py	2011-07-01 11:27:27 +0000
@@ -87,7 +87,7 @@
     @cachedproperty
     def short_product_tag_cloud(self):
         """Show a preview of the product tag cloud."""
-        return BazaarProductView().products(
+        return BazaarProductView(None, None).products(
             num_products=config.launchpad.code_homepage_product_cloud_size)
 
 
@@ -150,7 +150,7 @@
         self.request.response.redirect(redirect_url, status=301)
 
 
-class BazaarProductView:
+class BazaarProductView(LaunchpadView):
     """Browser class for products gettable with Bazaar."""
 
     def _make_distribution_map(self, values, percentile_map):

=== modified file 'lib/lp/codehosting/codeimport/tests/test_uifactory.py'
--- lib/lp/codehosting/codeimport/tests/test_uifactory.py	2010-08-20 20:31:18 +0000
+++ lib/lp/codehosting/codeimport/tests/test_uifactory.py	2011-07-19 18:09:01 +0000
@@ -98,6 +98,16 @@
              'hi 3/10'],
             self.messages)
 
+    def test_user_warning(self):
+        factory = self.makeLoggingUIFactory()
+        factory.show_user_warning('cross_format_fetch',
+            from_format="athing", to_format="anotherthing")
+        message = factory._user_warning_templates['cross_format_fetch'] % {
+            "from_format": "athing",
+            "to_format": "anotherthing",
+            }
+        self.assertEqual([message], self.messages)
+
 
 def test_suite():
     return unittest.TestLoader().loadTestsFromName(__name__)

=== modified file 'lib/lp/codehosting/codeimport/tests/test_worker.py'
--- lib/lp/codehosting/codeimport/tests/test_worker.py	2011-06-21 05:38:15 +0000
+++ lib/lp/codehosting/codeimport/tests/test_worker.py	2011-07-20 17:20:03 +0000
@@ -24,12 +24,10 @@
     )
 from bzrlib.errors import (
     NoSuchFile,
-    NotBranchError,
     )
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib import trace
 from bzrlib.transport import get_transport
-from bzrlib.upgrade import upgrade
 from bzrlib.urlutils import (
     join as urljoin,
     local_path_from_url,
@@ -210,16 +208,17 @@
         store = self.makeBranchStore()
         target_url = store._getMirrorURL(self.arbitrary_branch_id)
         knit_format = format_registry.get('knit')()
-        create_branch_with_one_revision(target_url, format=knit_format)
-        default_format = BzrDirFormat.get_default_format()
+        tree = create_branch_with_one_revision(target_url, format=knit_format)
+        self.assertNotEquals(tree.bzrdir._format.repository_format.network_name(),
+            default_format.repository_format.network_name())
 
         # The fetched branch is in the default format.
         new_branch = store.pull(
             self.arbitrary_branch_id, self.temp_dir, default_format)
         # Make sure backup.bzr is removed, as it interferes with CSCVS.
         self.assertEquals(os.listdir(self.temp_dir), [".bzr"])
-        self.assertEqual(
-            default_format, new_branch.bzrdir._format)
+        self.assertEquals(new_branch.repository._format.network_name(),
+            default_format.repository_format.network_name())
 
     def test_pushUpgradesFormat(self):
         # A branch should always be in the most up-to-date format before a
@@ -228,7 +227,6 @@
         target_url = store._getMirrorURL(self.arbitrary_branch_id)
         knit_format = format_registry.get('knit')()
         create_branch_with_one_revision(target_url, format=knit_format)
-        default_format = BzrDirFormat.get_default_format()
 
         # The fetched branch is in the default format.
         new_branch = store.pull(

=== modified file 'lib/lp/codehosting/codeimport/uifactory.py'
--- lib/lp/codehosting/codeimport/uifactory.py	2010-08-20 20:31:18 +0000
+++ lib/lp/codehosting/codeimport/uifactory.py	2011-07-19 18:09:01 +0000
@@ -36,9 +36,13 @@
         """
         TextUIFactory.__init__(self)
         self.interval = interval
+        self.writer = writer
         self._progress_view = LoggingTextProgressView(
             time_source, writer, interval)
 
+    def show_user_warning(self, warning_id, **message_args):
+        self.writer(self.format_user_warning(warning_id, message_args))
+
 
 class LoggingTextProgressView(TextProgressView):
     """Support class for `LoggingUIFactory`. """

=== modified file 'lib/lp/codehosting/codeimport/worker.py'
--- lib/lp/codehosting/codeimport/worker.py	2011-06-21 05:38:15 +0000
+++ lib/lp/codehosting/codeimport/worker.py	2011-07-20 17:20:03 +0000
@@ -100,15 +100,6 @@
             if needs_tree:
                 local_branch.bzrdir.create_workingtree()
             return local_branch
-        # XXX Tim Penhey 2009-09-18 bug 432217 Automatic upgrade of import
-        # branches disabled.  Need an orderly upgrade process.
-        if False and remote_bzr_dir.needs_format_conversion(
-            format=required_format):
-            try:
-                remote_bzr_dir.root_transport.delete_tree('backup.bzr')
-            except NoSuchFile:
-                pass
-            upgrade(remote_url, required_format)
         # The proper thing to do here would be to call
         # "remote_bzr_dir.sprout()".  But 2a fetch slowly checks which
         # revisions are in the ancestry of the tip of the remote branch, which
@@ -121,6 +112,12 @@
         target_control.create_prefix()
         remote_bzr_dir.transport.copy_tree_to_transport(target_control)
         local_bzr_dir = BzrDir.open_from_transport(target)
+        if local_bzr_dir.needs_format_conversion(format=required_format):
+            try:
+                local_bzr_dir.root_transport.delete_tree('backup.bzr')
+            except NoSuchFile:
+                pass
+            upgrade(target_path, required_format, clean_up=True)
         if needs_tree:
             local_bzr_dir.create_workingtree()
         return local_bzr_dir.open_branch()

=== modified file 'lib/lp/registry/adapters.py'
--- lib/lp/registry/adapters.py	2011-03-31 12:53:27 +0000
+++ lib/lp/registry/adapters.py	2011-07-25 19:21:49 +0000
@@ -9,6 +9,7 @@
     'distroseries_to_distribution',
     'PollSubset',
     'productseries_to_product',
+    'sourcepackage_to_distribution',
     ]
 
 
@@ -28,6 +29,14 @@
     )
 
 
+def sourcepackage_to_distribution(source_package):
+    """Adapts `ISourcePackage` object to `IDistribution`.
+
+    This also supports `IDistributionSourcePackage`
+    """
+    return source_package.distribution
+
+
 def distroseries_to_distribution(distroseries):
     """Adapts `IDistroSeries` object to `IDistribution`.
 

=== modified file 'lib/lp/registry/browser/codeofconduct.py'
--- lib/lp/registry/browser/codeofconduct.py	2010-11-23 23:22:27 +0000
+++ lib/lp/registry/browser/codeofconduct.py	2011-07-12 15:45:09 +0000
@@ -220,7 +220,7 @@
         self.request = request
 
 
-class SignedCodeOfConductAdminView:
+class SignedCodeOfConductAdminView(LaunchpadView):
     """Admin Console for SignedCodeOfConduct Entries."""
 
     def __init__(self, context, request):

=== modified file 'lib/lp/registry/browser/distributionsourcepackage.py'
--- lib/lp/registry/browser/distributionsourcepackage.py	2011-06-29 16:23:36 +0000
+++ lib/lp/registry/browser/distributionsourcepackage.py	2011-07-28 17:34:34 +0000
@@ -4,7 +4,6 @@
 __metaclass__ = type
 
 __all__ = [
-    'distribution_from_distributionsourcepackage',
     'DistributionSourcePackageAnswersMenu',
     'DistributionSourcePackageBreadcrumb',
     'DistributionSourcePackageChangelogView',
@@ -28,7 +27,6 @@
     getUtility,
     )
 from zope.interface import (
-    implementer,
     implements,
     Interface,
     )
@@ -116,12 +114,6 @@
             self.context.sourcepackagename.name)
 
 
-@adapter(IDistributionSourcePackage)
-@implementer(IServiceUsage)
-def distribution_from_distributionsourcepackage(dsp):
-    return dsp.distribution
-
-
 class DistributionSourcePackageFacets(QuestionTargetFacetMixin,
                                       StandardLaunchpadFacets):
 
@@ -130,7 +122,8 @@
 
     def overview(self):
         text = 'Overview'
-        summary = 'General information about {0}'.format(self.context.displayname)
+        summary = 'General information about {0}'.format(
+            self.context.displayname)
         return Link('', text, summary)
 
     def bugs(self):
@@ -277,7 +270,7 @@
         return Link('+changelog', text, icon="info")
 
 
-class DistributionSourcePackageBaseView:
+class DistributionSourcePackageBaseView(LaunchpadView):
     """Common features to all `DistributionSourcePackage` views."""
 
     def releases(self):
@@ -633,7 +626,7 @@
     cancel_url = next_url
 
 
-class DistributionSourcePackageHelpView:
+class DistributionSourcePackageHelpView(LaunchpadView):
     """A View to show Answers help."""
 
     page_title = 'Help and support options'

=== modified file 'lib/lp/registry/browser/distroseries.py'
--- lib/lp/registry/browser/distroseries.py	2011-07-20 16:51:49 +0000
+++ lib/lp/registry/browser/distroseries.py	2011-07-28 18:35:44 +0000
@@ -747,26 +747,26 @@
 
 # A helper to create package filtering radio button vocabulary.
 NON_IGNORED = 'non-ignored'
-IGNORED = 'ignored'
 HIGHER_VERSION_THAN_PARENT = 'higher-than-parent'
 RESOLVED = 'resolved'
+ALL = 'all'
 
 DEFAULT_PACKAGE_TYPE = NON_IGNORED
 
 
 def make_package_type_vocabulary(parent_name, higher_version_option=False):
     voc = [
-        SimpleTerm(
-            NON_IGNORED, NON_IGNORED, 'Non ignored packages'),
-        SimpleTerm(IGNORED, IGNORED, 'Ignored packages'),
-        SimpleTerm(RESOLVED, RESOLVED, "Resolved package differences")]
+        SimpleTerm(NON_IGNORED, NON_IGNORED, 'Non ignored packages'),
+        SimpleTerm(RESOLVED, RESOLVED, "Resolved package differences"),
+        SimpleTerm(ALL, ALL, 'All packages'),
+        ]
     if higher_version_option:
         higher_term = SimpleTerm(
             HIGHER_VERSION_THAN_PARENT,
             HIGHER_VERSION_THAN_PARENT,
             "Ignored packages with a higher version than in %s"
                 % parent_name)
-        voc.insert(2, higher_term)
+        voc.insert(1, higher_term)
     return SimpleVocabulary(tuple(voc))
 
 
@@ -808,6 +808,7 @@
 
     # Differences type to display. Can be overrided by sublasses.
     differences_type = DistroSeriesDifferenceType.DIFFERENT_VERSIONS
+    show_parent = True
     show_parent_version = True
     show_derived_version = True
     show_package_diffs = True
@@ -1058,10 +1059,10 @@
         package_type_dsd_status = {
             NON_IGNORED: (
                 DistroSeriesDifferenceStatus.NEEDS_ATTENTION,),
-            IGNORED: DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT,
             HIGHER_VERSION_THAN_PARENT: (
                 DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT),
             RESOLVED: DistroSeriesDifferenceStatus.RESOLVED,
+            ALL: DistroSeriesDifferenceStatus.items,
         }
 
         status = package_type_dsd_status[self.specified_package_type]
@@ -1252,7 +1253,8 @@
     """
     page_title = 'Unique packages'
     differences_type = DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES
-    show_parent_version = False
+    show_parent = True
+    show_parent_version = False  # The DSDs are unique to the derived series.
     show_package_diffs = False
     show_packagesets = True
 

=== modified file 'lib/lp/registry/browser/peoplemerge.py'
--- lib/lp/registry/browser/peoplemerge.py	2011-05-27 21:12:25 +0000
+++ lib/lp/registry/browser/peoplemerge.py	2011-07-27 17:34:09 +0000
@@ -139,7 +139,7 @@
                 naked_email.personID = self.target_person.id
                 naked_email.accountID = self.target_person.accountID
                 naked_email.status = EmailAddressStatus.NEW
-        job = getUtility(IPersonSet).mergeAsync(
+        getUtility(IPersonSet).mergeAsync(
             self.dupe_person, self.target_person, reviewer=self.user)
         self.request.response.addInfoNotification(self.merge_message)
         self.next_url = self.success_url
@@ -214,7 +214,6 @@
 
         super(AdminTeamMergeView, self).validate(data)
         dupe_team = data['dupe_person']
-        target_team = data['target_person']
         # We cannot merge the teams if there is a mailing list on the
         # duplicate person, unless that mailing list is purged.
         if self.hasMailingList(dupe_team):
@@ -342,19 +341,18 @@
             return ''
 
 
-class RequestPeopleMergeMultipleEmailsView:
+class RequestPeopleMergeMultipleEmailsView(LaunchpadView):
     """Merge request view when dupe account has multiple email addresses."""
 
     label = 'Merge Launchpad accounts'
     page_title = label
 
     def __init__(self, context, request):
-        self.context = context
-        self.request = request
+        super(RequestPeopleMergeMultipleEmailsView, self).__init__(
+            context, request)
         self.form_processed = False
         self.dupe = None
         self.notified_addresses = []
-        self.user = getUtility(ILaunchBag).user
 
     def processForm(self):
         dupe = self.request.form.get('dupe')

=== modified file 'lib/lp/registry/browser/sourcepackage.py'
--- lib/lp/registry/browser/sourcepackage.py	2011-06-29 16:23:36 +0000
+++ lib/lp/registry/browser/sourcepackage.py	2011-07-28 17:34:34 +0000
@@ -41,7 +41,6 @@
     )
 from zope.formlib.form import Fields
 from zope.interface import (
-    implementer,
     implements,
     Interface,
     )
@@ -84,7 +83,6 @@
     )
 from lp.app.browser.tales import CustomizableFormatter
 from lp.app.enums import ServiceUsage
-from lp.app.interfaces.launchpad import IServiceUsage
 from lp.app.widgets.itemswidgets import LaunchpadRadioWidget
 from lp.bugs.browser.bugtask import BugTargetTraversalMixin
 from lp.registry.browser.product import ProjectAddStepOne
@@ -190,12 +188,6 @@
 
 
 @adapter(ISourcePackage)
-@implementer(IServiceUsage)
-def distribution_from_sourcepackage(package):
-    return package.distribution
-
-
-@adapter(ISourcePackage)
 class SourcePackageBreadcrumb(Breadcrumb):
     """Builds a breadcrumb for an `ISourcePackage`."""
     implements(IBreadcrumb)
@@ -212,7 +204,8 @@
 
     def overview(self):
         text = 'Overview'
-        summary = 'General information about {0}'.format(self.context.displayname)
+        summary = 'General information about {0}'.format(
+            self.context.displayname)
         return Link('', text, summary)
 
     def bugs(self):
@@ -227,7 +220,8 @@
 
     def translations(self):
         text = 'Translations'
-        summary = 'Translations of {0} in Launchpad'.format(self.context.displayname)
+        summary = 'Translations of {0} in Launchpad'.format(
+            self.context.displayname)
         return Link('', text, summary)
 
 
@@ -444,7 +438,7 @@
                 'The packaging link has already been deleted.')
 
 
-class SourcePackageView:
+class SourcePackageView(LaunchpadView):
     """A view for (distro series) source packages."""
 
     def initialize(self):

=== modified file 'lib/lp/registry/browser/teammembership.py'
--- lib/lp/registry/browser/teammembership.py	2011-05-27 21:12:25 +0000
+++ lib/lp/registry/browser/teammembership.py	2011-07-28 14:19:11 +0000
@@ -25,7 +25,6 @@
     LaunchpadView,
     )
 from canonical.launchpad.webapp.breadcrumb import Breadcrumb
-from canonical.launchpad.webapp.interfaces import ILaunchBag
 from lp.app.errors import UnexpectedFormData
 from lp.app.interfaces.launchpad import ILaunchpadCelebrities
 from lp.app.widgets.date import DateWidget
@@ -40,12 +39,10 @@
         return "%s's membership" % self.context.person.displayname
 
 
-class TeamMembershipEditView:
+class TeamMembershipEditView(LaunchpadView):
 
     def __init__(self, context, request):
-        self.context = context
-        self.request = request
-        self.user = getUtility(ILaunchBag).user
+        super(TeamMembershipEditView, self).__init__(context, request)
         self.errormessage = ""
         self.prefix = 'membership'
         self.max_year = 2050
@@ -256,7 +253,6 @@
 
         assert self.context.status == TeamMembershipStatus.PROPOSED
 
-        action = self.request.form.get('editproposed')
         if self.request.form.get('decline'):
             status = TeamMembershipStatus.DECLINED
         elif self.request.form.get('approve'):

=== modified file 'lib/lp/registry/browser/tests/test_distroseries.py'
--- lib/lp/registry/browser/tests/test_distroseries.py	2011-07-25 11:56:30 +0000
+++ lib/lp/registry/browser/tests/test_distroseries.py	2011-07-28 18:35:44 +0000
@@ -50,7 +50,7 @@
 from lp.archivepublisher.debversion import Version
 from lp.registry.browser.distroseries import (
     HIGHER_VERSION_THAN_PARENT,
-    IGNORED,
+    ALL,
     NON_IGNORED,
     RESOLVED,
     seriesToVocab,
@@ -1611,27 +1611,23 @@
         self.assertContentEqual(
             [diff2, diff1], filtered_view2.cached_differences.batch)
 
-    def test_batch_differences_packages(self):
-        # field.package_type parameter allows to list only
-        # blacklisted differences.
+    def test_batch_all_packages(self):
+        # field.package_type parameter allows to list all the
+        # differences.
         set_derived_series_ui_feature_flag(self)
         derived_series, parent_series = self._createChildAndParent()
-        blacklisted_diff = self.factory.makeDistroSeriesDifference(
-            derived_series=derived_series,
-            status=DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT)
-
-        blacklisted_view = create_initialized_view(
+        # Create differences of all possible statuses.
+        diffs = []
+        for status in DistroSeriesDifferenceStatus.items:
+            diff = self.factory.makeDistroSeriesDifference(
+                derived_series=derived_series, status=status)
+            diffs.append(diff)
+        all_view = create_initialized_view(
             derived_series,
             '+localpackagediffs',
-            query_string='field.package_type=%s' % IGNORED)
-        unblacklisted_view = create_initialized_view(
-            derived_series,
-            '+localpackagediffs')
+            query_string='field.package_type=%s' % ALL)
 
-        self.assertContentEqual(
-            [blacklisted_diff], blacklisted_view.cached_differences.batch)
-        self.assertContentEqual(
-            [], unblacklisted_view.cached_differences.batch)
+        self.assertContentEqual(diffs, all_view.cached_differences.batch)
 
     def test_batch_blacklisted_differences_with_higher_version(self):
         # field.package_type parameter allows to list only
@@ -2337,6 +2333,35 @@
         self.assertContentEqual(
             [missing_diff], view.cached_differences.batch)
 
+    def test_uniquepackages_displays_parent(self):
+        # For a series derived from multiple parents, the parent for each
+        # DSD is displayed; no parent version is displayed because we're
+        # listing packages unique to the derived series.
+        set_derived_series_ui_feature_flag(self)
+        derived_series, parent_series = self._createChildAndParents()
+        missing_type = DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES
+        self.factory.makeDistroSeriesDifference(
+            difference_type=missing_type,
+            derived_series=derived_series,
+            parent_series=parent_series,
+            status=DistroSeriesDifferenceStatus.NEEDS_ATTENTION)
+        view = create_initialized_view(
+            derived_series, '+uniquepackages')
+
+        multiple_parents_display_matcher = soupmatchers.HTMLContains(
+            soupmatchers.Tag(
+                "Parent table header", 'th',
+                text=re.compile("^\s*Parent\s")),
+            Not(soupmatchers.Tag(
+                "Parent version table header", 'th',
+                text=re.compile("\s*Parent version\s*"))),
+            soupmatchers.Tag(
+                "Parent name", 'a',
+                attrs={'class': 'parent-name'},
+                text=re.compile("\s*%s\s*" % parent_series.displayname)),
+             )
+        self.assertThat(view.render(), multiple_parents_display_matcher)
+
     def test_uniquepackages_differences_empty(self):
         # The view is empty if there is no differences with type
         # UNIQUE_TO_DERIVED_SERIES.

=== modified file 'lib/lp/registry/browser/tests/test_subscription_links.py'
--- lib/lp/registry/browser/tests/test_subscription_links.py	2011-06-22 14:09:43 +0000
+++ lib/lp/registry/browser/tests/test_subscription_links.py	2011-07-01 11:27:27 +0000
@@ -69,7 +69,7 @@
             None, self.new_edit_link,
             "Expected edit_bug_mail link missing")
         # Ensure the LP.cache has been populated.
-        self.assertTrue("LP.cache['administratedTeams']" in self.contents)
+        self.assertIn('LP.cache = {"administratedTeams": [', self.contents)
         # Ensure the call to setup the subscription is in the HTML.
         # Only check for the presence of setup's configuration step; more
         # detailed checking is needlessly brittle.

=== modified file 'lib/lp/registry/configure.zcml'
--- lib/lp/registry/configure.zcml	2011-06-20 05:56:54 +0000
+++ lib/lp/registry/configure.zcml	2011-07-25 18:50:46 +0000
@@ -257,6 +257,11 @@
             interface="lp.registry.interfaces.distroseries.IDistroSeriesSet"/>
     </securedutility>
     <adapter
+        provides="lp.registry.interfaces.distribution.IDistribution"
+        for="lp.registry.interfaces.distroseries.IDistroSeries"
+        factory="lp.registry.adapters.distroseries_to_distribution"
+        permission="zope.Public"/>
+    <adapter
         provides="lp.app.interfaces.launchpad.ILaunchpadUsage"
         for="lp.registry.interfaces.distroseries.IDistroSeries"
         factory="lp.registry.adapters.distroseries_to_distribution"
@@ -532,11 +537,19 @@
                 "/>
     </class>
     <adapter
+        provides="lp.registry.interfaces.distribution.IDistribution"
+        for="lp.registry.interfaces.distributionsourcepackage.IDistributionSourcePackage"
+        factory="lp.registry.adapters.sourcepackage_to_distribution"
+        permission="zope.Public"/>
+    <adapter
         for="lp.registry.interfaces.distributionsourcepackage.IDistributionSourcePackage"
         provides="canonical.launchpad.webapp.interfaces.ILaunchpadContainer"
         factory="canonical.launchpad.components.launchpadcontainer.LaunchpadDistributionSourcePackageContainer"/>
     <adapter
-        factory="lp.registry.browser.distributionsourcepackage.distribution_from_distributionsourcepackage"/>
+        provides="lp.app.interfaces.launchpad.IServiceUsage"
+        for="lp.registry.interfaces.distributionsourcepackage.IDistributionSourcePackage"
+        factory="lp.registry.adapters.sourcepackage_to_distribution"
+        permission="zope.Public"/>
     <adapter
         factory="lp.registry.browser.distributionsourcepackage.DistributionSourcePackageBreadcrumb"/>
 
@@ -1614,9 +1627,17 @@
             interface="lp.registry.interfaces.sourcepackage.ISourcePackageFactory"/>
     </securedutility>
     <adapter
+        provides="lp.registry.interfaces.distribution.IDistribution"
+        for="lp.registry.interfaces.sourcepackage.ISourcePackage"
+        factory="lp.registry.adapters.sourcepackage_to_distribution"
+        permission="zope.Public"/>
+    <adapter
         factory="lp.registry.browser.sourcepackage.SourcePackageBreadcrumb"/>
     <adapter
-        factory="lp.registry.browser.sourcepackage.distribution_from_sourcepackage"/>
+        provides="lp.app.interfaces.launchpad.IServiceUsage"
+        for="lp.registry.interfaces.sourcepackage.ISourcePackage"
+        factory="lp.registry.adapters.sourcepackage_to_distribution"
+        permission="zope.Public"/>
 
     <!-- This is class ProductReleaseSet -->
 

=== modified file 'lib/lp/registry/errors.py'
--- lib/lp/registry/errors.py	2011-06-16 20:12:00 +0000
+++ lib/lp/registry/errors.py	2011-07-27 08:13:18 +0000
@@ -5,7 +5,6 @@
 __all__ = [
     'DistroSeriesDifferenceError',
     'NotADerivedSeriesError',
-    'MultipleParentsForDerivedSeriesError',
     'CannotTransitionToCountryMirror',
     'CountryMirrorAlreadySet',
     'DeleteSubscriptionError',
@@ -124,14 +123,6 @@
     non-derived series - that is, a distroseries with a null Parent."""
 
 
-class MultipleParentsForDerivedSeriesError(Exception):
-    """A distro series difference must have one parent series set.
-
-    This is raised when a DistroSeriesDifference is created, and we can't
-    determine the parent series to use, since there is more than one. The
-    parent series needs to be specified in this case."""
-
-
 @error_status(httplib.BAD_REQUEST)
 class TeamMembershipTransitionError(ValueError):
     """Indicates something has gone wrong with the transtiion.

=== modified file 'lib/lp/registry/interfaces/distroseriesdifference.py'
--- lib/lp/registry/interfaces/distroseriesdifference.py	2011-07-08 09:06:24 +0000
+++ lib/lp/registry/interfaces/distroseriesdifference.py	2011-07-27 08:13:18 +0000
@@ -272,7 +272,7 @@
 class IDistroSeriesDifferenceSource(Interface):
     """A utility of this interface can be used to create differences."""
 
-    def new(derived_series, source_package_name, parent_series=None):
+    def new(derived_series, source_package_name, parent_series):
         """Create an `IDistroSeriesDifference`.
 
         :param derived_series: The distribution series which was derived
@@ -283,8 +283,7 @@
             package with a difference.
         :type source_package_name: `ISourcePackageName`.
         :param parent_series: The distribution series which has the derived
-            series as a child. If there is only one parent, it does not need
-            to be specified.
+            series as a child.
         :type parent_series: `IDistroSeries`.
         :raises NotADerivedSeriesError: When the passed distro series
             is not a derived series.

=== modified file 'lib/lp/registry/interfaces/person.py'
--- lib/lp/registry/interfaces/person.py	2011-05-31 03:49:23 +0000
+++ lib/lp/registry/interfaces/person.py	2011-07-22 17:30:41 +0000
@@ -1400,16 +1400,20 @@
     @operation_parameters(
         name=TextLine(required=True, constraint=name_validator),
         displayname=TextLine(required=False),
-        description=TextLine(required=False))
+        description=TextLine(required=False),
+        private=Bool(required=False),
+        )
     @export_factory_operation(Interface, [])  # Really IArchive.
     @operation_for_version("beta")
-    def createPPA(name=None, displayname=None, description=None):
+    def createPPA(name=None, displayname=None, description=None, private=False):
         """Create a PPA.
 
         :param name: A string with the name of the new PPA to create. If
             not specified, defaults to 'ppa'.
         :param displayname: The displayname for the new PPA.
         :param description: The description for the new PPA.
+        :param private: Whether or not to create a private PPA. Defaults to
+            False, which means the PPA will be public.
         :raises: `PPACreationError` if an error is encountered
 
         :return: a PPA `IArchive` record.

=== modified file 'lib/lp/registry/model/distroseriesdifference.py'
--- lib/lp/registry/model/distroseriesdifference.py	2011-07-27 08:27:09 +0000
+++ lib/lp/registry/model/distroseriesdifference.py	2011-07-28 18:37:47 +0000
@@ -20,7 +20,6 @@
     )
 from lazr.enum import DBItem
 from sqlobject import StringCol
-from storm.exceptions import NotOneError
 from storm.expr import (
     And,
     Column,
@@ -55,7 +54,6 @@
     )
 from lp.registry.errors import (
     DistroSeriesDifferenceError,
-    MultipleParentsForDerivedSeriesError,
     NotADerivedSeriesError,
     )
 from lp.registry.interfaces.distroseriesdifference import (
@@ -407,22 +405,13 @@
     base_version = StringCol(dbName='base_version', notNull=False)
 
     @staticmethod
-    def new(derived_series, source_package_name, parent_series=None):
+    def new(derived_series, source_package_name, parent_series):
         """See `IDistroSeriesDifferenceSource`."""
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-        # mandatory as part of multi-parent support.
-        if parent_series is None:
-            try:
-                dsps = getUtility(IDistroSeriesParentSet)
-                dsp = dsps.getByDerivedSeries(
-                    derived_series).one()
-            except NotOneError:
-                raise MultipleParentsForDerivedSeriesError()
-            else:
-                if dsp is None:
-                    raise NotADerivedSeriesError()
-                else:
-                    parent_series = dsp.parent_series
+        dsps = getUtility(IDistroSeriesParentSet)
+        dsp = dsps.getByDerivedAndParentSeries(
+            derived_series, parent_series)
+        if dsp is None:
+            raise NotADerivedSeriesError()
 
         store = IMasterStore(DistroSeriesDifference)
         diff = DistroSeriesDifference()
@@ -439,15 +428,10 @@
         return store.add(diff)
 
     @staticmethod
-    def getForDistroSeries(
-        distro_series,
-        difference_type=None,
-        name_filter=None,
-        status=None,
-        child_version_higher=False,
-        parent_series=None,
-        packagesets=None,
-        changed_by=None):
+    def getForDistroSeries(distro_series, difference_type=None,
+                           name_filter=None, status=None,
+                           child_version_higher=False, parent_series=None,
+                           packagesets=None, changed_by=None):
         """See `IDistroSeriesDifferenceSource`."""
         if difference_type is None:
             difference_type = DistroSeriesDifferenceType.DIFFERENT_VERSIONS

=== modified file 'lib/lp/registry/model/person.py'
--- lib/lp/registry/model/person.py	2011-06-16 20:12:00 +0000
+++ lib/lp/registry/model/person.py	2011-07-27 19:13:40 +0000
@@ -2739,17 +2739,17 @@
         """See `IPerson`."""
         return getUtility(IArchiveSet).getPPAOwnedByPerson(self, name)
 
-    def createPPA(self, name=None, displayname=None, description=None):
+    def createPPA(self, name=None, displayname=None, description=None,
+                  private=False):
         """See `IPerson`."""
-        errors = Archive.validatePPA(self, name)
+        errors = Archive.validatePPA(self, name, private)
         if errors:
             raise PPACreationError(errors)
         ubuntu = getUtility(ILaunchpadCelebrities).ubuntu
-        ppa = getUtility(IArchiveSet).new(
+        return getUtility(IArchiveSet).new(
             owner=self, purpose=ArchivePurpose.PPA,
             distribution=ubuntu, name=name, displayname=displayname,
             description=description)
-        return ppa
 
     def isBugContributor(self, user=None):
         """See `IPerson`."""

=== modified file 'lib/lp/registry/templates/distroseries-localdifferences.pt'
--- lib/lp/registry/templates/distroseries-localdifferences.pt	2011-07-26 10:27:34 +0000
+++ lib/lp/registry/templates/distroseries-localdifferences.pt	2011-07-28 10:56:43 +0000
@@ -45,7 +45,7 @@
           <thead>
             <tr>
               <th>Source</th>
-              <th tal:condition="python: not(view.has_unique_parent) and view.show_parent_version">
+              <th tal:condition="python: not(view.has_unique_parent) and view.show_parent">
                 Parent
               </th>
               <th tal:condition="view/show_parent_version">
@@ -106,7 +106,7 @@
                   <span class="lowlight" tal:content="structure activity"></span>
                 </tal:activity>
               </td>
-              <td tal:condition="python: not(view.has_unique_parent) and view.show_parent_version">
+              <td tal:condition="python: not(view.has_unique_parent) and view.show_parent">
                 <a tal:attributes="href difference/parent_series/fmt:url"
                    tal:content="difference/parent_series/displayname"
                    class="parent-name">Warty</a>

=== modified file 'lib/lp/registry/templates/packagesearch-macros.pt'
--- lib/lp/registry/templates/packagesearch-macros.pt	2011-03-25 03:26:29 +0000
+++ lib/lp/registry/templates/packagesearch-macros.pt	2011-07-28 13:47:10 +0000
@@ -63,7 +63,7 @@
     class="distroseries-localdiff-search-filter"
     action="" method="GET">
     <div style="float:left; margin-right:1px;">
-      <label for="field.name_filter">Show packages with names matching:</label>
+      <label for="field.name_filter">Show packages with names or packagesets matching:</label>
     </div>
     <div style="float:left;">
       <input

=== modified file 'lib/lp/registry/tests/test_adapters.py'
--- lib/lp/registry/tests/test_adapters.py	2011-03-14 12:20:17 +0000
+++ lib/lp/registry/tests/test_adapters.py	2011-07-25 19:21:49 +0000
@@ -9,6 +9,7 @@
 from lp.registry.adapters import (
     distroseries_to_distribution,
     productseries_to_product,
+    sourcepackage_to_distribution,
     )
 from lp.registry.interfaces.distribution import IDistribution
 from lp.registry.interfaces.product import IProduct
@@ -19,13 +20,32 @@
 
     layer = DatabaseFunctionalLayer
 
+    def test_sourcepackage_to_distribution_dsp(self):
+        # A distribution can be retrieved from a dsp.
+        package = self.factory.makeDistributionSourcePackage()
+        distribution = sourcepackage_to_distribution(package)
+        self.assertTrue(IDistribution.providedBy(distribution))
+        self.assertEqual(package.distribution, distribution)
+        self.assertEqual(package.distribution, IDistribution(package))
+
+    def test_sourcepackage_to_distribution_sp(self):
+        # A distribution can be retrieved from a source package.
+        package = self.factory.makeSourcePackage()
+        distribution = sourcepackage_to_distribution(package)
+        self.assertTrue(IDistribution.providedBy(distribution))
+        self.assertEqual(package.distroseries.distribution, distribution)
+        self.assertEqual(
+            package.distroseries.distribution, IDistribution(package))
+
     def test_distroseries_to_distribution(self):
         # distroseries_to_distribution() returns an IDistribution given an
         # IDistroSeries.
         distro_series = self.factory.makeDistroSeries()
         distribution = distroseries_to_distribution(distro_series)
         self.assertTrue(IDistribution.providedBy(distribution))
-        self.assertEqual(distribution, distro_series.distribution)
+        self.assertEqual(distro_series.distribution, distribution)
+        self.assertEqual(
+            distro_series.distribution, IDistribution(distro_series))
 
     def test_productseries_to_product(self):
         # productseries_to_product() returns an IProduct given an
@@ -33,4 +53,4 @@
         product_series = self.factory.makeProductSeries()
         product = productseries_to_product(product_series)
         self.assertTrue(IProduct.providedBy(product))
-        self.assertEqual(product, product_series.product)
+        self.assertEqual(product_series.product, product)

=== modified file 'lib/lp/registry/tests/test_distroseriesdifference.py'
--- lib/lp/registry/tests/test_distroseriesdifference.py	2011-07-26 13:58:36 +0000
+++ lib/lp/registry/tests/test_distroseriesdifference.py	2011-07-28 18:37:47 +0000
@@ -74,7 +74,8 @@
 
         self.assertRaises(
             NotADerivedSeriesError, distroseriesdifference_factory.new,
-            distro_series, source_package_name)
+            distro_series, source_package_name,
+            self.factory.makeDistroSeries())
 
     def test_source_pub(self):
         # The related source pub is returned for the derived series.
@@ -836,7 +837,7 @@
         # factory.makeDistroSeriesDifference() will always create
         # publications to be helpful. We don't need the help in this case.
         dsd = getUtility(IDistroSeriesDifferenceSource).new(
-            dsp.derived_series, spn)
+            dsp.derived_series, spn, dsp.parent_series)
         self.assertEqual(pd, dsd.package_diff)
 
     def _initDiffWithMultiplePendingPublications(self, versions, parent):

=== modified file 'lib/lp/registry/tests/test_dsp_vocabularies.py'
--- lib/lp/registry/tests/test_dsp_vocabularies.py	2011-07-22 01:15:49 +0000
+++ lib/lp/registry/tests/test_dsp_vocabularies.py	2011-07-26 19:06:06 +0000
@@ -12,8 +12,7 @@
 
 
 class TestDistributionSourcePackageVocabulary(TestCaseWithFactory):
-    """Test that the DistributionSourcePackageVocabulary behaves as
-    expected."""
+    """Test that the vocabulary behaves as expected."""
     layer = DatabaseFunctionalLayer
 
     def test_provides_ihugevocabulary(self):
@@ -21,19 +20,140 @@
             self.factory.makeDistribution())
         self.assertProvides(vocabulary, IHugeVocabulary)
 
-    def test_toTerm_unbuilt_dsp(self):
-        # If the source has no built binaries, the term's value contains a
-        # string to that effect.
-        dsp = self.factory.makeDistributionSourcePackage(
-            sourcepackagename='foo')
-        vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
-        term = vocabulary.toTerm(dsp.name)
-        self.assertEqual(dsp.sourcepackagename.name, term.title)
-        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
-        self.assertEqual(expected_token, term.token)
-        self.assertEqual('Not yet built.', term.value)
-
-    def test_toTerm_built_single_binary(self):
+    def test_init_IDistribution(self):
+        # When the context is adaptable to IDistribution, it also provides
+        # the distribution.
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename='foo')
+        vocabulary = DistributionSourcePackageVocabulary(dsp)
+        self.assertEqual(dsp, vocabulary.context)
+        self.assertEqual(dsp.distribution, vocabulary.distribution)
+
+    def test_init_dsp_bugtask(self):
+        # A dsp bugtask can be the context
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename='foo')
+        bugtask = self.factory.makeBugTask(target=dsp)
+        vocabulary = DistributionSourcePackageVocabulary(bugtask)
+        self.assertEqual(bugtask, vocabulary.context)
+        self.assertEqual(dsp.distribution, vocabulary.distribution)
+
+    def test_init_dsp_question(self):
+        # A dsp bugtask can be the context
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename='foo')
+        question = self.factory.makeQuestion(
+            target=dsp, owner=dsp.distribution.owner)
+        vocabulary = DistributionSourcePackageVocabulary(question)
+        self.assertEqual(question, vocabulary.context)
+        self.assertEqual(dsp.distribution, vocabulary.distribution)
+
+    def test_init_no_distribution(self):
+        # The distribution is None if the context cannot be adapted to a
+        # distribution.
+        project = self.factory.makeProduct()
+        vocabulary = DistributionSourcePackageVocabulary(project)
+        self.assertEqual(project, vocabulary.context)
+        self.assertEqual(None, vocabulary.distribution)
+
+    def test_getDistributionAndPackageName_distro_and_package(self):
+        # getDistributionAndPackageName() returns a tuple of distribution
+        # and package name when the text contains both.
+        new_distro = self.factory.makeDistribution(name='fnord')
+        vocabulary = DistributionSourcePackageVocabulary(None)
+        distribution, package_name = vocabulary.getDistributionAndPackageName(
+            'fnord/pting')
+        self.assertEqual(new_distro, distribution)
+        self.assertEqual('pting', package_name)
+
+    def test_getDistributionAndPackageName_default_distro_and_package(self):
+        # getDistributionAndPackageName() returns a tuple of the default
+        # distribution and package name when the text is just a package name.
+        default_distro = self.factory.makeDistribution(name='fnord')
+        vocabulary = DistributionSourcePackageVocabulary(default_distro)
+        distribution, package_name = vocabulary.getDistributionAndPackageName(
+            'pting')
+        self.assertEqual(default_distro, distribution)
+        self.assertEqual('pting', package_name)
+
+    def test_getDistributionAndPackageName_bad_distro_and_package(self):
+        # getDistributionAndPackageName() returns a tuple of the default
+        # distribution and package name when the distro in the text cannot
+        # be matched to a real distro.
+        default_distro = self.factory.makeDistribution(name='fnord')
+        vocabulary = DistributionSourcePackageVocabulary(default_distro)
+        distribution, package_name = vocabulary.getDistributionAndPackageName(
+            'misspelled/pting')
+        self.assertEqual(default_distro, distribution)
+        self.assertEqual('pting', package_name)
+
+    def test_contains_true(self):
+        # The vocabulary contains DSPs that have SPPH in the distro.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp)
+        self.assertTrue(dsp in vocabulary)
+
+    def test_contains_false(self):
+        # The vocabulary does not contain DSPs without SPPH.
+        spn = self.factory.makeSourcePackageName(name='foo')
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename=spn)
+        vocabulary = DistributionSourcePackageVocabulary(dsp)
+        self.assertFalse(dsp in vocabulary)
+
+    def test_toTerm_raises_error(self):
+        # An error is raised for DSP/SPNs without publishing history.
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename='foo')
+        vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
+        self.assertRaises(LookupError, vocabulary.toTerm, dsp.name)
+
+    def test_toTerm_none_raises_error(self):
+        # An error is raised for SPN does not exist.
+        vocabulary = DistributionSourcePackageVocabulary(None)
+        self.assertRaises(LookupError, vocabulary.toTerm, 'non-existant')
+
+    def test_toTerm_spn_and_default_distribution(self):
+        # The vocabulary's distribution is used when only a SPN is passed.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
+        term = vocabulary.toTerm(dsp.sourcepackagename)
+        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
+        self.assertEqual(expected_token, term.token)
+        self.assertEqual(dsp.sourcepackagename, term.value)
+
+    def test_toTerm_spn_and_distribution(self):
+        # The distribution is used with the spn if it is passed.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(None)
+        term = vocabulary.toTerm(dsp.sourcepackagename, dsp.distribution)
+        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
+        self.assertEqual(expected_token, term.token)
+        self.assertEqual(dsp.sourcepackagename, term.value)
+
+    def test_toTerm_dsp(self):
+        # The DSP's distribution is used when a DSP is passed.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp)
+        term = vocabulary.toTerm(dsp)
+        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
+        self.assertEqual(expected_token, term.token)
+        self.assertEqual(dsp.sourcepackagename, term.value)
+
+    def test_toTerm_unbuilt_title(self):
+        # The DSP's distribution is used with the spn if it is passed.
+        # Published, but unbuilt packages state the case in the term title.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp)
+        term = vocabulary.toTerm(dsp)
+        self.assertEqual('Not yet built.', term.title)
+
+    def test_toTerm_built_single_binary_title(self):
         # The binary package name appears in the term's value.
         bpph = self.factory.makeBinaryPackagePublishingHistory()
         spr = bpph.binarypackagerelease.build.source_package_release
@@ -42,11 +162,9 @@
             distribution=bpph.distroseries.distribution)
         vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
         term = vocabulary.toTerm(spr.sourcepackagename)
-        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
-        self.assertEqual(expected_token, term.token)
-        self.assertEqual(bpph.binary_package_name, term.value)
+        self.assertEqual(bpph.binary_package_name, term.title)
 
-    def test_toTerm_built_multiple_binary(self):
+    def test_toTerm_built_multiple_binary_title(self):
         # All of the binary package names appear in the term's value.
         spph = self.factory.makeSourcePackagePublishingHistory()
         spr = spph.sourcepackagerelease
@@ -63,16 +181,40 @@
         dsp = spr.distrosourcepackage
         vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
         term = vocabulary.toTerm(spr.sourcepackagename)
-        expected_token = '%s/%s' % (dsp.distribution.name, dsp.name)
-        self.assertEqual(expected_token, term.token)
-        self.assertEqual(', '.join(expected_names), term.value)
+        self.assertEqual(', '.join(expected_names), term.title)
+
+    def test_getTermByToken_error(self):
+        # An error is raised if the token does not match a published DSP.
+        dsp = self.factory.makeDistributionSourcePackage(
+            sourcepackagename='foo')
+        vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
+        token = '%s/%s' % (dsp.distribution.name, dsp.name)
+        self.assertRaises(LookupError, vocabulary.getTermByToken, token)
+
+    def test_getTermByToken_token(self):
+        # The term is return if it matches a published DSP.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp.distribution)
+        token = '%s/%s' % (dsp.distribution.name, dsp.name)
+        term = vocabulary.getTermByToken(token)
+        self.assertEqual(dsp.sourcepackagename, term.value)
+
+    def test_searchForTerms_without_distribution(self):
+        # An empty result set is return if the vocabulary has no distribution
+        # and the search does not provide distribution information.
+        spph = self.factory.makeSourcePackagePublishingHistory()
+        dsp = spph.sourcepackagerelease.distrosourcepackage
+        vocabulary = DistributionSourcePackageVocabulary(dsp.name)
+        results = vocabulary.searchForTerms(dsp.name)
+        self.assertIs(0, results.count())
 
     def test_searchForTerms_None(self):
         # Searching for nothing gets you that.
         vocabulary = DistributionSourcePackageVocabulary(
             self.factory.makeDistribution())
         results = vocabulary.searchForTerms()
-        self.assertIs(None, results)
+        self.assertIs(0, results.count())
 
     def assertTermsEqual(self, expected, actual):
         # Assert two given terms are equal.
@@ -125,7 +267,7 @@
         spn = self.factory.getOrMakeSourcePackageName('xorg')
         spr = self.factory.makeSourcePackageRelease(sourcepackagename=spn)
         das = self.factory.makeDistroArchSeries()
-        spph = self.factory.makeSourcePackagePublishingHistory(
+        self.factory.makeSourcePackagePublishingHistory(
             sourcepackagerelease=spr, distroseries=das.distroseries)
         for name in ('xorg-common', 'xorg-server', 'xorg-video-intel'):
             bpn = self.factory.getOrMakeBinaryPackageName(name)
@@ -133,7 +275,7 @@
                 source_package_release=spr, distroarchseries=das)
             bpr = self.factory.makeBinaryPackageRelease(
                 binarypackagename=bpn, build=bpb)
-            bpph = self.factory.makeBinaryPackagePublishingHistory(
+            self.factory.makeBinaryPackagePublishingHistory(
                 binarypackagerelease=bpr, distroarchseries=das)
         vocabulary = DistributionSourcePackageVocabulary(
             context=das.distroseries.distribution)

=== modified file 'lib/lp/registry/vocabularies.py'
--- lib/lp/registry/vocabularies.py	2011-07-22 09:33:33 +0000
+++ lib/lp/registry/vocabularies.py	2011-07-26 19:09:48 +0000
@@ -80,6 +80,7 @@
     With,
     )
 from storm.info import ClassAlias
+from storm.store import EmptyResultSet
 from zope.component import getUtility
 from zope.interface import implements
 from zope.schema.interfaces import IVocabularyTokenized
@@ -129,7 +130,10 @@
 from lp.app.interfaces.launchpad import ILaunchpadCelebrities
 from lp.blueprints.interfaces.specification import ISpecification
 from lp.bugs.interfaces.bugtask import IBugTask
-from lp.registry.interfaces.distribution import IDistribution
+from lp.registry.interfaces.distribution import (
+    IDistribution,
+    IDistributionSet,
+    )
 from lp.registry.interfaces.distributionsourcepackage import (
     IDistributionSourcePackage,
     )
@@ -634,7 +638,7 @@
                     Person.id.is_in(public_inner_textual_select),
                     Person.visibility == PersonVisibility.PUBLIC,
                     Person.merged == None,
-                    Or(# A valid person-or-team is either a team...
+                    Or(  # A valid person-or-team is either a team...
                        # Note: 'Not' due to Bug 244768.
                        Not(Person.teamowner == None),
                        # Or a person who has a preferred email address.
@@ -830,10 +834,10 @@
                 And(
                     SQL("Person.id = MatchingPerson.id"),
                     Or(
-                        And(# A public person or team
+                        And(  # A public person or team
                             Person.visibility == PersonVisibility.PUBLIC,
                             Person.merged == None,
-                            Or(# A valid person-or-team is either a team...
+                            Or(  # A valid person-or-team is either a team...
                                 # Note: 'Not' due to Bug 244768.
                                 Not(Person.teamowner == None),
                                 # Or a person who has preferred email address.
@@ -1979,13 +1983,27 @@
 
     implements(IHugeVocabulary)
     displayname = 'Select a package'
-    step_title = 'Search'
+    step_title = 'Search by name or distro/name'
 
     def __init__(self, context):
         self.context = context
+        # Avoid circular import issues.
+        from lp.answers.interfaces.question import IQuestion
+        if IBugTask.providedBy(context) or IQuestion.providedBy(context):
+            target = context.target
+        else:
+            target = context
+        try:
+            self.distribution = IDistribution(target)
+        except TypeError:
+            self.distribution = None
 
-    def __contains__(self, obj):
-        pass
+    def __contains__(self, spn_or_dsp):
+        try:
+            self.toTerm(spn_or_dsp)
+            return True
+        except LookupError:
+            return False
 
     def __iter__(self):
         pass
@@ -1993,31 +2011,59 @@
     def __len__(self):
         pass
 
-    def toTerm(self, spn):
+    def getDistributionAndPackageName(self, text):
+        "Return the distribution and package name from the parsed text."
+        # Match the toTerm() format, but also use it to select a distribution.
+        distribution = None
+        if '/' in text:
+            distro_name, text = text.split('/', 1)
+            distribution = getUtility(IDistributionSet).getByName(distro_name)
+        if distribution is None:
+            distribution = self.distribution
+        return distribution, text
+
+    def toTerm(self, spn_or_dsp, distribution=None):
         """See `IVocabulary`."""
-        dsp = self.context.getSourcePackage(spn)
-        if dsp.publishing_history:
+        dsp = None
+        if IDistributionSourcePackage.providedBy(spn_or_dsp):
+            dsp = spn_or_dsp
+            distribution = spn_or_dsp.distribution
+        else:
+            distribution = distribution or self.distribution
+            if distribution is not None and spn_or_dsp is not None:
+                dsp = distribution.getSourcePackage(spn_or_dsp)
+        try:
             binaries = dsp.publishing_history[0].getBuiltBinaries()
-            summary = ', '.join(
-                [binary.binary_package_name for binary in binaries])
-        else:
-            summary = "Not yet built."
-        token = '%s/%s' % (dsp.distribution.name, dsp.name)
-        return SimpleTerm(summary, token, dsp.name)
+            binary_names = [binary.binary_package_name for binary in binaries]
+            if binary_names != []:
+                summary = ', '.join(binary_names)
+            else:
+                summary = 'Not yet built.'
+            token = '%s/%s' % (dsp.distribution.name, dsp.name)
+            return SimpleTerm(dsp.sourcepackagename, token, summary)
+        except (IndexError, AttributeError):
+            # Either the DSP was None or there is no publishing history.
+            raise LookupError(distribution, spn_or_dsp)
 
-    def getTerm(self, spn):
+    def getTerm(self, spn_or_dsp):
         """See `IBaseVocabulary`."""
-        return self.toTerm(spn)
+        return self.toTerm(spn_or_dsp)
 
     def getTermByToken(self, token):
         """See `IVocabularyTokenized`."""
-        pass
+        distribution, package_name = self.getDistributionAndPackageName(token)
+        return self.toTerm(package_name, distribution)
 
     def searchForTerms(self, query=None):
         """See `IHugeVocabulary`."""
-        distribution = self.context
-        if query is None:
-            return
+        if not query:
+            return EmptyResultSet()
+        distribution, query = self.getDistributionAndPackageName(query)
+        if distribution is None:
+            # This could failover to ubuntu, but that is non-obvious. The
+            # Python widget must set the default distribution and the JS
+            # widget must encourage the <distro>/<package> search format.
+            return EmptyResultSet()
         search_term = unicode(query)
         store = IStore(SourcePackagePublishingHistory)
         spns = store.using(
@@ -2057,4 +2103,5 @@
                     SourcePackageName.name.contains_string(search_term),
                     BinaryPackageName.name.contains_string(
                         search_term))).config(distinct=True)
+        # XXX sinzui 2011-07-26: This query ignored SPN branches.
         return CountableIterator(spns.count(), spns, self.toTerm)

=== modified file 'lib/lp/soyuz/browser/distroarchseriesbinarypackage.py'
--- lib/lp/soyuz/browser/distroarchseriesbinarypackage.py	2010-08-20 20:31:18 +0000
+++ lib/lp/soyuz/browser/distroarchseriesbinarypackage.py	2011-07-27 17:34:09 +0000
@@ -11,8 +11,8 @@
 from canonical.launchpad.webapp import (
     ApplicationMenu,
     GetitemNavigation,
+    LaunchpadView,
     )
-from canonical.launchpad.webapp.breadcrumb import Breadcrumb
 from canonical.lazr.utils import smartquote
 from lp.soyuz.interfaces.distroarchseriesbinarypackage import (
     IDistroArchSeriesBinaryPackage,
@@ -31,11 +31,7 @@
     usedfor = IDistroArchSeriesBinaryPackage
 
 
-class DistroArchSeriesBinaryPackageView:
-
-    def __init__(self, context, request):
-        self.context = context
-        self.request = request
+class DistroArchSeriesBinaryPackageView(LaunchpadView):
 
     @property
     def page_title(self):

=== modified file 'lib/lp/soyuz/browser/distroarchseriesbinarypackagerelease.py'
--- lib/lp/soyuz/browser/distroarchseriesbinarypackagerelease.py	2010-08-20 20:31:18 +0000
+++ lib/lp/soyuz/browser/distroarchseriesbinarypackagerelease.py	2011-07-27 17:34:09 +0000
@@ -11,6 +11,7 @@
 
 from canonical.launchpad.webapp import (
     ApplicationMenu,
+    LaunchpadView,
     Navigation,
     )
 from canonical.launchpad.webapp.breadcrumb import Breadcrumb
@@ -38,9 +39,8 @@
     usedfor = IDistroArchSeriesBinaryPackageRelease
 
 
-class DistroArchSeriesBinaryPackageReleaseView:
+class DistroArchSeriesBinaryPackageReleaseView(LaunchpadView):
 
     def __init__(self, context, request):
         self.context = context
         self.request = request
-

=== modified file 'lib/lp/soyuz/browser/distroseriesbinarypackage.py'
--- lib/lp/soyuz/browser/distroseriesbinarypackage.py	2010-08-20 20:31:18 +0000
+++ lib/lp/soyuz/browser/distroseriesbinarypackage.py	2011-07-01 11:27:27 +0000
@@ -12,6 +12,7 @@
 
 from canonical.launchpad.webapp import (
     ApplicationMenu,
+    LaunchpadView,
     Navigation,
     StandardLaunchpadFacets,
     )
@@ -49,7 +50,7 @@
         return self.context.binarypackagename.name
 
 
-class DistroSeriesBinaryPackageView:
+class DistroSeriesBinaryPackageView(LaunchpadView):
 
     def __init__(self, context, request):
         self.context = context

=== modified file 'lib/lp/soyuz/browser/distroseriessourcepackagerelease.py'
--- lib/lp/soyuz/browser/distroseriessourcepackagerelease.py	2010-08-20 20:31:18 +0000
+++ lib/lp/soyuz/browser/distroseriessourcepackagerelease.py	2011-07-01 11:27:27 +0000
@@ -10,6 +10,7 @@
 
 from canonical.launchpad.webapp import (
     ApplicationMenu,
+    LaunchpadView,
     Navigation,
     stepthrough,
     )
@@ -49,7 +50,7 @@
         return None
 
 
-class DistroSeriesSourcePackageReleaseView:
+class DistroSeriesSourcePackageReleaseView(LaunchpadView):
 
     def __init__(self, context, request):
         self.context = context

=== modified file 'lib/lp/soyuz/browser/queue.py'
--- lib/lp/soyuz/browser/queue.py	2011-07-26 08:51:39 +0000
+++ lib/lp/soyuz/browser/queue.py	2011-07-28 07:59:23 +0000
@@ -27,6 +27,7 @@
     load_referencing,
     load_related,
     )
+from lp.services.job.model.job import Job
 from lp.soyuz.enums import (
     PackagePublishingPriority,
     PackageUploadStatus,
@@ -190,6 +191,19 @@
         return getUtility(IPackagesetSet).getForPackages(
             self.context, set(spr.sourcepackagenameID for spr in sprs))
 
+    def loadPackageCopyJobs(self, uploads):
+        """Batch-load `PackageCopyJob`s and related information."""
+        # Avoid circular imports.
+        from lp.registry.model.person import Person
+        from lp.soyuz.model.archive import Archive
+        from lp.soyuz.model.packagecopyjob import PackageCopyJob
+
+        package_copy_jobs = load_related(
+            PackageCopyJob, uploads, ['package_copy_job_id'])
+        load_related(Archive, package_copy_jobs, ['source_archive_id'])
+        jobs = load_related(Job, package_copy_jobs, ['job_id'])
+        load_related(Person, jobs, ['requester_id'])
+
     def decoratedQueueBatch(self):
         """Return the current batch, converted to decorated objects.
 
@@ -245,6 +259,8 @@
 
         package_sets = self.getPackagesetsFor(source_sprs)
 
+        self.loadPackageCopyJobs(uploads)
+
         return [
             CompletePackageUpload(
                 item, build_upload_files, source_upload_files, package_sets)

=== modified file 'lib/lp/soyuz/browser/sourcepackage.py'
--- lib/lp/soyuz/browser/sourcepackage.py	2011-05-12 12:09:17 +0000
+++ lib/lp/soyuz/browser/sourcepackage.py	2011-07-01 11:27:27 +0000
@@ -12,7 +12,10 @@
 
 from zope.component import getUtility
 
-from canonical.launchpad.webapp import Navigation
+from canonical.launchpad.webapp import (
+    LaunchpadView,
+    Navigation,
+    )
 from canonical.lazr.utils import smartquote
 from lp.registry.interfaces.distribution import IDistributionSet
 from lp.registry.interfaces.distroseries import IDistroSeriesSet
@@ -21,7 +24,7 @@
     )
 
 
-class SourcePackageChangelogView:
+class SourcePackageChangelogView(LaunchpadView):
     """View class for source package change logs."""
 
     page_title = "Change log"
@@ -32,7 +35,7 @@
         return smartquote("Change logs for " + self.context.title)
 
 
-class SourcePackageCopyrightView:
+class SourcePackageCopyrightView(LaunchpadView):
     """A view to display a source package's copyright information."""
 
     page_title = "Copyright"

=== modified file 'lib/lp/soyuz/browser/tests/test_queue.py'
--- lib/lp/soyuz/browser/tests/test_queue.py	2011-06-23 13:05:52 +0000
+++ lib/lp/soyuz/browser/tests/test_queue.py	2011-07-28 07:59:23 +0000
@@ -236,6 +236,11 @@
             view = self.makeView(upload.distroseries, queue_admin)
             html_text = view()
         self.assertIn(upload.package_name, html_text)
+        # The details section states the sync's origin and requester.
+        self.assertIn(
+            upload.package_copy_job.source_archive.displayname, html_text)
+        self.assertIn(
+            upload.package_copy_job.job.requester.displayname, html_text)
 
 
 class TestCompletePackageUpload(TestCaseWithFactory):

=== modified file 'lib/lp/soyuz/configure.zcml'
--- lib/lp/soyuz/configure.zcml	2011-07-14 15:23:28 +0000
+++ lib/lp/soyuz/configure.zcml	2011-07-28 14:46:35 +0000
@@ -181,6 +181,7 @@
                 is_delayed_copy
                 isPPA
                 package_copy_job
+                package_copy_job_id
                 package_name
                 package_version
                 section_name
@@ -405,6 +406,12 @@
             permission="launchpad.Edit"
             interface="lp.soyuz.interfaces.archive.IArchiveEdit"
             set_attributes="description displayname publish status"/>
+        <!--
+           NOTE: The 'private' permission controls who can turn a public
+           archive into a private one, and vice versa. The logic that says this
+           requires launchpad.Commercial permissions is duplicated in
+           IArchive.validatePPA.
+          -->
         <require
             permission="launchpad.Commercial"
 	    interface="lp.soyuz.interfaces.archive.IArchiveCommercial"

=== modified file 'lib/lp/soyuz/interfaces/distributionjob.py'
--- lib/lp/soyuz/interfaces/distributionjob.py	2011-07-22 11:17:04 +0000
+++ lib/lp/soyuz/interfaces/distributionjob.py	2011-07-27 10:34:53 +0000
@@ -121,8 +121,7 @@
 class IDistroSeriesDifferenceJobSource(IJobSource):
     """An `IJob` for creating `DistroSeriesDifference`s."""
 
-    def createForPackagePublication(derivedseries, sourcepackagename, pocket,
-                                    parent_series=None):
+    def createForPackagePublication(derivedseries, sourcepackagename, pocket):
         """Create jobs as appropriate for a given status publication.
 
         :param derived_series: A `DistroSeries` that is assumed to be
@@ -130,22 +129,15 @@
         :param sourcepackagename: A `SourcePackageName` that is being
             published in `derived_series` or `parent_series`.
         :param pocket: The `PackagePublishingPocket` for the publication.
-        :param parent_series: The parent `DistroSeries` whose version of
-            `sourcepackagename` is to be compared with that in
-            `derived_series`.
         :return: An iterable of `DistroSeriesDifferenceJob`.
         """
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-        # mandatory as part of multi-parent support.
 
-    def massCreateForSeries(derived_series, parent_series):
+    def massCreateForSeries(derived_series):
         """Create jobs for all the publications inside the given distroseries
             with reference to the given parent series.
 
         :param derived_series: A `DistroSeries` that is assumed to be
             derived from `parent_series`.
-        :param parent_series: The parent `DistroSeries`.
-
         :return: An iterable of `DistroSeriesDifferenceJob` ids. We don't
             return the Job themselves for performance reason.
         """

=== modified file 'lib/lp/soyuz/model/archive.py'
--- lib/lp/soyuz/model/archive.py	2011-07-22 13:28:35 +0000
+++ lib/lp/soyuz/model/archive.py	2011-07-28 09:26:39 +0000
@@ -1947,8 +1947,18 @@
         self.enabled_restricted_families = restricted
 
     @classmethod
-    def validatePPA(self, person, proposed_name):
+    def validatePPA(self, person, proposed_name, private=False):
         ubuntu = getUtility(ILaunchpadCelebrities).ubuntu
+        if private:
+            # NOTE: This duplicates the policy in lp/soyuz/configure.zcml
+            # which says that one needs 'launchpad.Commercial' permission to
+            # set 'private', and the logic in `AdminByCommercialTeamOrAdmins`
+            # which determines who is granted launchpad.Commercial
+            # permissions.
+            commercial = getUtility(ILaunchpadCelebrities).commercial_admin
+            admin = getUtility(ILaunchpadCelebrities).admin
+            if not person.inTeam(commercial) and not person.inTeam(admin):
+                return '%s is not allowed to make private PPAs' % (person.name,)
         if person.isTeam() and (
             person.subscriptionpolicy in OPEN_TEAM_POLICY):
             return "Open teams cannot have PPAs."

=== modified file 'lib/lp/soyuz/model/distroseriesdifferencejob.py'
--- lib/lp/soyuz/model/distroseriesdifferencejob.py	2011-07-22 11:12:23 +0000
+++ lib/lp/soyuz/model/distroseriesdifferencejob.py	2011-07-27 10:34:53 +0000
@@ -48,21 +48,15 @@
 FEATURE_FLAG_ENABLE_MODULE = u"soyuz.derived_series_jobs.enabled"
 
 
-def make_metadata(sourcepackagename, parent_series=None):
+def make_metadata(sourcepackagename, parent_series):
     """Return JSON metadata for a job on `sourcepackagename`."""
-    # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-    # mandatory as part of multi-parent support.
-    if parent_series is None:
-        parent_id = None
-    else:
-        parent_id = parent_series.id
     return {
         'sourcepackagename': sourcepackagename.id,
-        'parent_series': parent_id,
+        'parent_series': parent_series.id,
     }
 
 
-def create_job(derived_series, sourcepackagename, parent_series=None):
+def create_job(derived_series, sourcepackagename, parent_series):
     """Create a `DistroSeriesDifferenceJob` for a given source package.
 
     :param derived_series: A `DistroSeries` that is assumed to be derived
@@ -73,8 +67,6 @@
         `derived_series`.  The difference is between the versions of
         `sourcepackagename` in `parent_series` and `derived_series`.
     """
-    # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-    # mandatory as part of multi-parent support.
     job = DistributionJob(
         distribution=derived_series.distribution, distroseries=derived_series,
         job_type=DistributionJobType.DISTROSERIESDIFFERENCE,
@@ -85,13 +77,12 @@
 
 def create_multiple_jobs(derived_series, parent_series):
     """Create a `DistroSeriesDifferenceJob` for all the source packages in
-    archive (optionally limited to the sourcepackagenames passed).
+    archive.
 
     :param derived_series: A `DistroSeries` that is assumed to be derived
         from another one.
     :param parent_series: A `DistroSeries` that is a parent of
         `derived_series`.
-    :param archive: A `IArchive` where to find the source packages.
     """
     store = IStore(SourcePackageRelease)
     source_package_releases = store.find(
@@ -134,7 +125,7 @@
     return [job_id for job_id, in result]
 
 
-def find_waiting_jobs(derived_series, sourcepackagename, parent_series=None):
+def find_waiting_jobs(derived_series, sourcepackagename, parent_series):
     """Look for pending `DistroSeriesDifference` jobs on a package."""
     # Look for identical pending jobs.  This compares directly on
     # the metadata string.  It's fragile, but this is only an
@@ -155,18 +146,13 @@
         DistributionJob._json_data == json_metadata,
         DistributionJob.job_id.is_in(Job.ready_jobs))
 
-    # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-    # mandatory as part of multi-parent support.
-    if parent_series is None:
-        return list(candidates)
-
     return [
         job
         for job in candidates
             if job.metadata["parent_series"] == parent_series.id]
 
 
-def may_require_job(derived_series, sourcepackagename, parent_series=None):
+def may_require_job(derived_series, sourcepackagename, parent_series):
     """Might publishing this package require a new job?
 
     Use this to determine whether to create a new
@@ -175,8 +161,6 @@
     runner some unnecessary work, but we don't expect a bit of
     unnecessary work to be a big problem.
     """
-    # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-    # mandatory as part of multi-parent support.
     if derived_series is None:
         return False
     dsp = getUtility(IDistroSeriesParentSet).getByDerivedSeries(
@@ -208,10 +192,8 @@
 
     @classmethod
     def createForPackagePublication(cls, derived_series, sourcepackagename,
-                                    pocket, parent_series=None):
+                                    pocket):
         """See `IDistroSeriesDifferenceJobSource`."""
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-        # mandatory as part of multi-parent support.
         if not getFeatureFlag(FEATURE_FLAG_ENABLE_MODULE):
             return
         # -backports and -proposed are not really part of a standard
@@ -223,19 +205,30 @@
             PackagePublishingPocket.PROPOSED):
             return
         jobs = []
-        children = list(derived_series.getDerivedSeries())
-        for relative in children + [derived_series]:
-            if may_require_job(relative, sourcepackagename, parent_series):
-                jobs.append(create_job(
-                    relative, sourcepackagename, parent_series))
+        parent_series = derived_series.getParentSeries()
+        # Create jobs for DSDs between the derived_series and its
+        # parents.
+        for parent in parent_series:
+            if may_require_job(
+                derived_series, sourcepackagename, parent):
+                jobs.append(create_job(
+                    derived_series, sourcepackagename, parent))
+        # Create jobs for DSDs between the derived_series and its
+        # children.
+        for child in derived_series.getDerivedSeries():
+            if may_require_job(
+                child, sourcepackagename, derived_series):
+                jobs.append(create_job(
+                    child, sourcepackagename, derived_series))
         return jobs
 
     @classmethod
-    def massCreateForSeries(cls, derived_series, parent_series):
+    def massCreateForSeries(cls, derived_series):
         """See `IDistroSeriesDifferenceJobSource`."""
         if not getFeatureFlag(FEATURE_FLAG_ENABLE_MODULE):
             return
-        create_multiple_jobs(derived_series, parent_series)
+        for parent_series in derived_series.getParentSeries():
+            create_multiple_jobs(derived_series, parent_series)
 
     @classmethod
     def getPendingJobsForDifferences(cls, derived_series,
@@ -248,13 +241,15 @@
             Job._status.is_in(Job.PENDING_STATUSES),
             DistributionJob.distroseries == derived_series)
 
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Check for parent
-        # series once it becomes available.
+        parent_series_ids = set(
+            dsd.parent_series.id for dsd in distroseriesdifferences)
         keyed_dsds = dict(
             (dsd.source_package_name.id, dsd)
             for dsd in distroseriesdifferences)
         jobs_by_dsd = {}
         for job in jobs:
+            if job.metadata["parent_series"] not in parent_series_ids:
+                continue
             dsd = keyed_dsds.get(job.metadata["sourcepackagename"])
             if dsd is not None:
                 jobs_by_dsd.setdefault(dsd, []).append(cls(job))
@@ -271,10 +266,6 @@
     @property
     def parent_series(self):
         parent_id = self.metadata['parent_series']
-        if not parent_id:
-            # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-            # mandatory as part of multi-parent support.
-            return None
         return IStore(DistroSeries).get(DistroSeries, parent_id)
 
     def passesPackagesetFilter(self):
@@ -286,12 +277,6 @@
         """
         derived_series = self.derived_series
         parent_series = self.parent_series
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-        # mandatory as part of multi-parent support.
-        if parent_series is None:
-            dsp = getUtility(IDistroSeriesParentSet).getByDerivedSeries(
-                derived_series)
-            parent_series = dsp[0].parent_series
 
         sourcepackagename = self.sourcepackagename
         if has_package(derived_series, sourcepackagename):
@@ -316,18 +301,11 @@
         spn_id = self.metadata["sourcepackagename"]
         parent_id = self.metadata["parent_series"]
         store = IMasterStore(DistroSeriesDifference)
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Make parent_series
-        # mandatory as part of multi-parent support.
-        if parent_id is None:
-            match_parent = True
-        else:
-            match_parent = (
-                DistroSeriesDifference.parent_series_id == parent_id)
         search = store.find(
             DistroSeriesDifference,
             DistroSeriesDifference.derived_series == self.derived_series,
-            DistroSeriesDifference.source_package_name_id == spn_id,
-            match_parent)
+            DistroSeriesDifference.parent_series_id == parent_id,
+            DistroSeriesDifference.source_package_name_id == spn_id)
         return search.one()
 
     def run(self):

=== modified file 'lib/lp/soyuz/model/publishing.py'
--- lib/lp/soyuz/model/publishing.py	2011-07-27 23:06:20 +0000
+++ lib/lp/soyuz/model/publishing.py	2011-07-28 18:37:47 +0000
@@ -1500,9 +1500,8 @@
                     pocket)
         return pub
 
-    def getBuildsForSourceIds(
-        self, source_publication_ids, archive=None, build_states=None,
-        need_build_farm_job=False):
+    def getBuildsForSourceIds(self, source_publication_ids, archive=None,
+                              build_states=None, need_build_farm_job=False):
         """See `IPublishingSet`."""
         # Import Build and DistroArchSeries locally to avoid circular
         # imports, since that Build uses SourcePackagePublishingHistory

=== modified file 'lib/lp/soyuz/scripts/initialize_distroseries.py'
--- lib/lp/soyuz/scripts/initialize_distroseries.py	2011-07-25 15:53:38 +0000
+++ lib/lp/soyuz/scripts/initialize_distroseries.py	2011-07-27 08:13:18 +0000
@@ -293,8 +293,7 @@
 
     def _create_dsd_jobs(self):
         job_source = getUtility(IDistroSeriesDifferenceJobSource)
-        for parent in self.parents:
-            job_source.massCreateForSeries(self.distroseries, parent)
+        job_source.massCreateForSeries(self.distroseries)
 
     def _copy_configuration(self):
         self.distroseries.backports_not_automatic = any(

=== modified file 'lib/lp/soyuz/stories/webservice/xx-person-createppa.txt'
--- lib/lp/soyuz/stories/webservice/xx-person-createppa.txt	2011-02-13 22:54:48 +0000
+++ lib/lp/soyuz/stories/webservice/xx-person-createppa.txt	2011-07-27 19:13:40 +0000
@@ -30,3 +30,31 @@
   Status: 400 Bad Request
   ...
   A PPA cannot have the same name as its distribution.
+
+== Creating private PPAs ==
+
+Our PPA owner now has a single PPA.
+
+  >>> print_self_link_of_entries(webservice.get(
+  ...     ppa_owner['ppas_collection_link']).jsonBody())
+  http://api.launchpad.dev/beta/~.../+archive/ppa
+
+They aren't a commercial admin, so they cannot create private PPAs.
+
+  >>> print ppa_owner_webservice.named_post(
+  ...     ppa_owner['self_link'], 'createPPA', {}, name='whatever',
+  ...     displayname='My secret new PPA', description='Secretness!',
+  ...     private=True,
+  ...     )
+  HTTP/1.1 400 Bad Request
+  Status: 400
+  ...
+  ... is not allowed to make private PPAs
+
+After attempting and failing to create a private PPA, they still have the same
+single PPA they had at the beginning:
+
+  >>> print_self_link_of_entries(webservice.get(
+  ...     ppa_owner['ppas_collection_link']).jsonBody())
+  http://api.launchpad.dev/beta/~.../+archive/ppa
+

=== modified file 'lib/lp/soyuz/templates/distroseries-queue.pt'
--- lib/lp/soyuz/templates/distroseries-queue.pt	2011-07-18 09:23:10 +0000
+++ lib/lp/soyuz/templates/distroseries-queue.pt	2011-07-28 07:59:23 +0000
@@ -239,6 +239,23 @@
       </tr>
     </tal:copy>
 
+    <tal:sync condition="packageupload/package_copy_job">
+      <tr>
+        <td />
+        <td tal:condition="view/availableActions" />
+        <td colspan="7">
+          Sync from
+          <tal:archive
+            content="packageupload/package_copy_job/source_archive/displayname">
+            Primary Archive for Ubuntu
+          </tal:archive>,
+          requested by
+          <tal:requester
+            content="structure packageupload/package_copy_job/job/requester/fmt:link" />
+        </td>
+      </tr>
+    </tal:sync>
+
     <tal:upload condition="not: packageupload/pending_delayed_copy">
       <tr tal:repeat="file packageupload/source_files">
         <td/>

=== modified file 'lib/lp/soyuz/tests/test_archive.py'
--- lib/lp/soyuz/tests/test_archive.py	2011-07-22 13:31:35 +0000
+++ lib/lp/soyuz/tests/test_archive.py	2011-07-27 19:31:01 +0000
@@ -1624,7 +1624,7 @@
             set(archive.getComponentsForUploader(person)))
 
 
-class TestvalidatePPA(TestCaseWithFactory):
+class TestValidatePPA(TestCaseWithFactory):
 
     layer = DatabaseFunctionalLayer
 
@@ -1639,6 +1639,30 @@
             'A PPA cannot have the same name as its distribution.',
             Archive.validatePPA(ppa_owner, 'ubuntu'))
 
+    def test_private_ppa_non_commercial_admin(self):
+        ppa_owner = self.factory.makePerson()
+        self.assertEqual(
+            '%s is not allowed to make private PPAs' % (ppa_owner.name,),
+            Archive.validatePPA(ppa_owner, self.factory.getUniqueString(),
+                                private=True))
+
+    def test_private_ppa_commercial_admin(self):
+        ppa_owner = self.factory.makePerson()
+        with celebrity_logged_in('admin'):
+            comm = getUtility(ILaunchpadCelebrities).commercial_admin
+            comm.addMember(ppa_owner, comm.teamowner)
+        self.assertIs(
+            None,
+            Archive.validatePPA(ppa_owner, self.factory.getUniqueString(),
+                                private=True))
+
+    def test_private_ppa_admin(self):
+        ppa_owner = self.factory.makeAdministrator()
+        self.assertIs(
+            None,
+            Archive.validatePPA(ppa_owner, self.factory.getUniqueString(),
+                                private=True))
+
     def test_two_ppas(self):
         ppa = self.factory.makeArchive(name='ppa')
         self.assertEqual("You already have a PPA named 'ppa'.",

=== modified file 'lib/lp/soyuz/tests/test_distroseriesdifferencejob.py'
--- lib/lp/soyuz/tests/test_distroseriesdifferencejob.py	2011-07-22 21:06:30 +0000
+++ lib/lp/soyuz/tests/test_distroseriesdifferencejob.py	2011-07-28 09:03:35 +0000
@@ -109,11 +109,6 @@
         package = self.factory.makeSourcePackageName()
         self.assertFalse(may_require_job(None, package, parent_series))
 
-    def test_may_require_job_accepts_none_parent_series(self):
-        derived_series = self.makeDerivedDistroSeries()
-        package = self.factory.makeSourcePackageName()
-        self.assertTrue(may_require_job(derived_series, package, None))
-
     def test_may_require_job_allows_new_jobs(self):
         dsp = self.factory.makeDistroSeriesParent()
         package = self.factory.makeSourcePackageName()
@@ -262,42 +257,54 @@
             [],
             find_waiting_jobs(dsp.derived_series, package, dsp.parent_series))
 
-    def test_createForPackagedPublication_creates_jobs_for_its_child(self):
-        dsp = self.factory.makeDistroSeriesParent()
-        parent_dsp = self.factory.makeDistroSeriesParent(
-            derived_series=dsp.parent_series)
-        package = self.factory.makeSourcePackageName()
-        # Create a job for the derived_series parent, which should create
-        # two jobs. One for derived_series, and the other for its child.
-        self.getJobSource().createForPackagePublication(
-            parent_dsp.derived_series, package,
-            PackagePublishingPocket.RELEASE, parent_dsp.parent_series)
-        jobs = sum([
-            find_waiting_jobs(dsp.derived_series, package, dsp.parent_series)
-            for dsp in [parent_dsp, parent_dsp]],
-            [])
+    def assertJobsSeriesAndMetadata(self, job, series, metadata):
+        self.assertEqual(job.distroseries, series)
         self.assertEqual(
-            [package.id, package.id],
-            [job.metadata["sourcepackagename"] for job in jobs])
+            (metadata[0], metadata[1]),
+            (job.metadata["sourcepackagename"],
+             job.metadata["parent_series"]))
 
     def test_createForPackagePublication_creates_job_for_derived_series(self):
+        # A call to createForPackagePublication for the derived_series
+        # creates a job for the derived series.
         dsp = self.factory.makeDistroSeriesParent()
+        parent_dsp = self.factory.makeDistroSeriesParent(
+            derived_series=dsp.parent_series)
         package = self.factory.makeSourcePackageName()
         self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.RELEASE,
-            dsp.parent_series)
+            parent_dsp.derived_series, package,
+            PackagePublishingPocket.RELEASE)
         jobs = find_waiting_jobs(
             dsp.derived_series, package, dsp.parent_series)
-        self.assertEqual(
-            [package.id], [job.metadata["sourcepackagename"] for job in jobs])
+
+        self.assertEquals(len(jobs), 1)
+        self.assertJobsSeriesAndMetadata(
+            jobs[0], dsp.derived_series, [package.id, dsp.parent_series.id])
+
+    def test_createForPackagePublication_creates_job_for_parent_series(self):
+        # A call to createForPackagePublication for the derived_series
+        # creates a job for the parent series.
+        dsp = self.factory.makeDistroSeriesParent()
+        parent_dsp = self.factory.makeDistroSeriesParent(
+            derived_series=dsp.parent_series)
+        package = self.factory.makeSourcePackageName()
+        self.getJobSource().createForPackagePublication(
+            parent_dsp.derived_series, package,
+            PackagePublishingPocket.RELEASE)
+        parent_jobs = find_waiting_jobs(
+            parent_dsp.derived_series, package, parent_dsp.parent_series)
+
+        self.assertEquals(len(parent_jobs), 1)
+        self.assertJobsSeriesAndMetadata(
+            parent_jobs[0], dsp.parent_series,
+            [package.id, parent_dsp.parent_series.id])
 
     def test_createForPackagePublication_obeys_feature_flag(self):
         dsp = self.factory.makeDistroSeriesParent()
         package = self.factory.makeSourcePackageName()
         self.useFixture(FeatureFixture({FEATURE_FLAG_ENABLE_MODULE: ''}))
         self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.RELEASE,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.RELEASE)
         self.assertContentEqual(
             [],
             find_waiting_jobs(dsp.derived_series, package, dsp.parent_series))
@@ -306,21 +313,18 @@
         dsp = self.factory.makeDistroSeriesParent()
         package = self.factory.makeSourcePackageName()
         self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.BACKPORTS,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.BACKPORTS)
         self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.PROPOSED,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.PROPOSED)
         self.assertContentEqual(
             [],
             find_waiting_jobs(dsp.derived_series, package, dsp.parent_series))
 
     def test_massCreateForSeries_obeys_feature_flag(self):
+        self.useFixture(FeatureFixture({FEATURE_FLAG_ENABLE_MODULE: ''}))
         dsp = self.factory.makeDistroSeriesParent()
         spph = self.createSPPHs(dsp.derived_series, 1)[0]
-        self.useFixture(FeatureFixture({FEATURE_FLAG_ENABLE_MODULE: ''}))
-        self.getJobSource().massCreateForSeries(
-            dsp.derived_series, dsp.parent_series)
+        self.getJobSource().massCreateForSeries(dsp.derived_series)
 
         self.assertContentEqual(
             [],
@@ -331,7 +335,8 @@
 
     def test_getPendingJobsForDifferences_finds_job(self):
         dsd = self.factory.makeDistroSeriesDifference()
-        job = create_job(dsd.derived_series, dsd.source_package_name)
+        job = create_job(
+            dsd.derived_series, dsd.source_package_name, dsd.parent_series)
         self.assertEqual(
             {dsd: [job]},
             self.getJobSource().getPendingJobsForDifferences(
@@ -339,7 +344,9 @@
 
     def test_getPendingJobsForDifferences_ignores_other_package(self):
         dsd = self.factory.makeDistroSeriesDifference()
-        create_job(dsd.derived_series, self.factory.makeSourcePackageName())
+        create_job(
+            dsd.derived_series, self.factory.makeSourcePackageName(),
+            dsd.parent_series)
         self.assertEqual(
             {},
             self.getJobSource().getPendingJobsForDifferences(
@@ -347,21 +354,29 @@
 
     def test_getPendingJobsForDifferences_ignores_other_derived_series(self):
         dsd = self.factory.makeDistroSeriesDifference()
-        create_job(self.makeDerivedDistroSeries(), dsd.source_package_name)
+        create_job(
+            self.makeDerivedDistroSeries(), dsd.source_package_name,
+            dsd.parent_series)
         self.assertEqual(
             {},
             self.getJobSource().getPendingJobsForDifferences(
                 dsd.derived_series, [dsd]))
 
     def test_getPendingJobsForDifferences_ignores_other_parent_series(self):
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Can't test this
-        # until we can specify the right parent series when creating a
-        # job.
-        return
+        dsd = self.factory.makeDistroSeriesDifference()
+        other_parent = self.factory.makeDistroSeriesParent(
+            dsd.derived_series).parent_series
+        create_job(
+            dsd.derived_series, dsd.source_package_name, other_parent)
+        self.assertEqual(
+            {},
+            self.getJobSource().getPendingJobsForDifferences(
+                dsd.derived_series, [dsd]))
 
     def test_getPendingJobsForDifferences_ignores_non_pending_jobs(self):
         dsd = self.factory.makeDistroSeriesDifference()
-        job = create_job(dsd.derived_series, dsd.source_package_name)
+        job = create_job(
+            dsd.derived_series, dsd.source_package_name, dsd.parent_series)
         removeSecurityProxy(job).job._status = JobStatus.COMPLETED
         self.assertEqual(
             {},
@@ -369,15 +384,14 @@
                 dsd.derived_series, [dsd]))
 
     def test_getPendingJobsForDifferences_ignores_other_job_types(self):
-        # XXX JeroenVermeulen 2011-05-26 bug=758906: Once parent_series
-        # is incorporated into the job type, set it to dsd.parent_series
-        # on the fake job, or this test will become silently meaningless.
         dsd = self.factory.makeDistroSeriesDifference()
         DistributionJob(
             distribution=dsd.derived_series.distribution,
             distroseries=dsd.derived_series,
             job_type=DistributionJobType.INITIALIZE_SERIES,
-            metadata={"sourcepackagename": dsd.source_package_name.id})
+            metadata={
+                "sourcepackagename": dsd.source_package_name.id,
+                "parent_series": dsd.parent_series.id})
         self.assertEqual(
             {},
             self.getJobSource().getPendingJobsForDifferences(
@@ -387,8 +401,7 @@
         dsp = self.factory.makeDistroSeriesParent()
         package = self.factory.makeSourcePackageName()
         self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.RELEASE,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.RELEASE)
         # Make changes visible to the process we'll be spawning.
         transaction.commit()
         return_code, stdout, stderr = run_script(
@@ -407,8 +420,7 @@
         dsp = self.factory.makeDistroSeriesParent()
         package = self.factory.makeSourcePackageName()
         job = self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.RELEASE,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.RELEASE)
         job[0].start()
         job[0].run()
         # Complete the job so we can create another.
@@ -417,8 +429,7 @@
         self.assertEqual(1, find_dsd_for(dsp, package).count())
         # If we run the job again, it will not create another DSD.
         job = self.getJobSource().createForPackagePublication(
-            dsp.derived_series, package, PackagePublishingPocket.RELEASE,
-            dsp.parent_series)
+            dsp.derived_series, package, PackagePublishingPocket.RELEASE)
         job[0].start()
         job[0].run()
         self.assertEqual(1, find_dsd_for(dsp, package).count())
@@ -561,7 +572,8 @@
 
         # Creating the SPPHs has created jobs for us, so grab them off
         # the queue.
-        jobs = find_waiting_jobs(dsp.derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            dsp.derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = find_dsd_for(dsp, source_package_name)
         self.assertEqual(1, ds_diff.count())
@@ -572,7 +584,8 @@
         self.createPublication(
             source_package_name, ['1.0-2', '1.0-1'],
             dsp.parent_series)
-        jobs = find_waiting_jobs(dsp.derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            dsp.derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         # And the DSD we have a hold of will have updated.
         self.assertEqual('1.0-2', ds_diff[0].parent_source_version)
@@ -588,7 +601,8 @@
             source_package_name, ['1.0-1'], dsp.derived_series)
         self.createPublication(
             source_package_name, ['1.0-1'], dsp.parent_series)
-        jobs = find_waiting_jobs(dsp.derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            dsp.derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = find_dsd_for(dsp, source_package_name)
         self.assertEqual(
@@ -596,7 +610,8 @@
         self.createPublication(
             source_package_name, ['2.0-0derived1', '1.0-1'],
             dsp.derived_series)
-        jobs = find_waiting_jobs(dsp.derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            dsp.derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         self.assertEqual(
             DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT,
@@ -607,7 +622,8 @@
         self.createPublication(
             source_package_name, ['2.0-0derived2', '1.0-1'],
             dsp.derived_series)
-        jobs = find_waiting_jobs(dsp.derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            dsp.derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         self.assertEqual(
             DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT,
@@ -623,13 +639,15 @@
             source_package_name, ['1.0-1derived1', '1.0-1'], derived_series)
         self.createPublication(
             source_package_name, ['1.0-2', '1.0-1'], dsp.parent_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         self.assertEqual('1.0-1', ds_diff[0].base_version)
         self.createPublication(
             source_package_name, ['1.0-2', '1.0-1'], derived_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         self.assertEqual(
             DistroSeriesDifferenceStatus.RESOLVED, ds_diff[0].status)
@@ -642,7 +660,8 @@
         source_package_name = self.factory.makeSourcePackageName()
         self.createPublication(
             source_package_name, ['1.0-0derived1'], derived_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         self.assertEqual(
@@ -657,7 +676,8 @@
         source_package_name = self.factory.makeSourcePackageName()
         self.createPublication(
             source_package_name, ['1.0-1'], dsp.parent_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         self.assertEqual(
@@ -674,13 +694,15 @@
             source_package_name, ['1.0-1'], derived_series)
         spph = self.createPublication(
             source_package_name, ['1.0-1'], dsp.parent_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         self.assertEqual(
             DistroSeriesDifferenceStatus.RESOLVED, ds_diff[0].status)
         spph.requestDeletion(self.factory.makePerson())
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         self.assertEqual(
             DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES,
@@ -696,13 +718,15 @@
             source_package_name, ['1.0-1'], derived_series)
         self.createPublication(
             source_package_name, ['1.0-1'], dsp.parent_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         self.assertEqual(
             DistroSeriesDifferenceStatus.RESOLVED, ds_diff[0].status)
         spph.requestDeletion(self.factory.makePerson())
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         self.assertEqual(
             DistroSeriesDifferenceType.MISSING_FROM_DERIVED_SERIES,
@@ -716,7 +740,9 @@
         self.createPublication(
             source_package_name, ['1.0-1'], dsp.derived_series, ppa)
         self.assertContentEqual(
-            [], find_waiting_jobs(dsp.derived_series, source_package_name))
+            [],
+            find_waiting_jobs(
+                dsp.derived_series, source_package_name, dsp.parent_series))
 
     def test_no_job_for_PPA_with_deleted_source(self):
         # If a source package is deleted from a PPA, no job is created.
@@ -728,7 +754,9 @@
             source_package_name, ['1.0-1'], derived_series, ppa)
         spph.requestDeletion(ppa.owner)
         self.assertContentEqual(
-            [], find_waiting_jobs(derived_series, source_package_name))
+            [],
+            find_waiting_jobs(
+                derived_series, source_package_name, dsp.parent_series))
 
     def test_update_deletes_diffs(self):
         # When a DSD is updated, the diffs are invalidated.
@@ -746,7 +774,8 @@
             archive=dsp.parent_series.main_archive,
             distroseries=dsp.parent_series,
             status=PackagePublishingStatus.SUPERSEDED)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         ds_diff = self.findDSD(derived_series, source_package_name)
         ds_diff[0].requestPackageDiffs(self.factory.makePerson())
@@ -755,7 +784,8 @@
         self.createPublication(
             source_package_name, ['1.0-3', '1.0-2', '1.0-1'],
             dsp.parent_series)
-        jobs = find_waiting_jobs(derived_series, source_package_name)
+        jobs = find_waiting_jobs(
+            derived_series, source_package_name, dsp.parent_series)
         self.runJob(jobs[0])
         # Since the diff showing the changes from 1.0-1 to 1.0-1derived1 is
         # still valid, it isn't reset, but the parent diff is.

=== modified file 'lib/lp/testing/__init__.py'
--- lib/lp/testing/__init__.py	2011-07-21 18:38:50 +0000
+++ lib/lp/testing/__init__.py	2011-07-27 13:41:54 +0000
@@ -15,6 +15,7 @@
     'build_yui_unittest_suite',
     'celebrity_logged_in',
     'ExpectedException',
+    'extract_lp_cache',
     'FakeTime',
     'get_lsb_information',
     'is_logged_in',
@@ -1306,3 +1307,8 @@
         self.caught_exc = exc_value
         return super(ExpectedException, self).__exit__(
             exc_type, exc_value, traceback)
+
+
+def extract_lp_cache(text):
+    match = re.search(r'<script>LP.cache = (\{.*\});</script>', text)
+    return simplejson.loads(match.group(1))

=== modified file 'lib/lp/testing/factory.py'
--- lib/lp/testing/factory.py	2011-07-24 22:30:54 +0000
+++ lib/lp/testing/factory.py	2011-07-26 17:18:19 +0000
@@ -2068,7 +2068,7 @@
             owner = target.owner
         if description is None:
             description = self.getUniqueString('description')
-        with person_logged_in(target.owner):
+        with person_logged_in(owner):
             question = target.newQuestion(
                 owner=owner, title=title, description=description)
         return question

=== modified file 'lib/lp/translations/browser/sourcepackage.py'
--- lib/lp/translations/browser/sourcepackage.py	2011-04-21 19:56:36 +0000
+++ lib/lp/translations/browser/sourcepackage.py	2011-07-01 11:27:27 +0000
@@ -43,7 +43,7 @@
 from lp.translations.model.translationpackagingjob import TranslationMergeJob
 
 
-class SourcePackageTranslationsView(TranslationsMixin,
+class SourcePackageTranslationsView(LaunchpadView, TranslationsMixin,
                                     TranslationSharingDetailsMixin):
 
     @property

=== modified file 'lib/lp/translations/browser/tests/test_sharing_details.py'
--- lib/lp/translations/browser/tests/test_sharing_details.py	2011-06-30 16:49:37 +0000
+++ lib/lp/translations/browser/tests/test_sharing_details.py	2011-07-15 14:10:46 +0000
@@ -26,6 +26,7 @@
 from lp.testing import (
     BrowserTestCase,
     EventRecorder,
+    extract_lp_cache,
     person_logged_in,
     TestCaseWithFactory,
     )
@@ -250,7 +251,7 @@
         # If the packaging link is set and if an upstream series
         # uses Launchpad translations but if the other conditions
         # are not fulfilled, is_configuration_complete is False.
-        self.configureSharing(translations_usage=ServiceUsage.LAUNCHPAD)
+        self.configureSharing(translations_usage = ServiceUsage.LAUNCHPAD)
         self.assertFalse(self.view.is_configuration_complete)
 
     def test_is_configuration_complete__no_auto_sync(self):
@@ -261,8 +262,8 @@
         # but if the upstream series does not synchronize translations
         # then is_configuration_complete is False.
         self.configureSharing(
-            set_upstream_branch=True,
-            translations_usage=ServiceUsage.LAUNCHPAD)
+            set_upstream_branch = True,
+            translations_usage = ServiceUsage.LAUNCHPAD)
         self.assertFalse(self.view.is_configuration_complete)
 
     def test_is_configuration_complete__all_conditions_fulfilled(self):
@@ -273,9 +274,9 @@
         #   - the upstream series synchronizes translations
         # then is_configuration_complete is True.
         self.configureSharing(
-            set_upstream_branch=True,
-            translations_usage=ServiceUsage.LAUNCHPAD,
-            translation_import_mode=
+            set_upstream_branch = True,
+            translations_usage = ServiceUsage.LAUNCHPAD,
+            translation_import_mode =
                 TranslationsBranchImportMode.IMPORT_TRANSLATIONS)
         self.assertTrue(self.view.is_configuration_complete)
 
@@ -938,10 +939,12 @@
         sourcepackage = self.makeFullyConfiguredSharing()[0]
         anon_browser = self._getSharingDetailsViewBrowser(sourcepackage)
         # Anonymous users don't get cached objects due to bug #740208
-        self.assertNotIn("LP.cache['productseries'] =", anon_browser.contents)
+        self.assertNotIn(
+            'productseries', extract_lp_cache(anon_browser.contents))
         browser = self._getSharingDetailsViewBrowser(
             sourcepackage, user=self.user)
-        self.assertIn("LP.cache['productseries'] =", browser.contents)
+        self.assertIn(
+            'productseries', extract_lp_cache(browser.contents))
 
     def test_potlist_only_ubuntu(self):
         # Without a packaging link, only Ubuntu templates are listed.

=== modified file 'lib/lp/translations/browser/translationgroup.py'
--- lib/lp/translations/browser/translationgroup.py	2010-11-23 23:22:27 +0000
+++ lib/lp/translations/browser/translationgroup.py	2011-07-12 15:50:08 +0000
@@ -57,7 +57,7 @@
     text = u"Translation groups"
 
 
-class TranslationGroupSetView:
+class TranslationGroupSetView(LaunchpadView):
     """Translation groups overview."""
     page_title = "Translation groups"
     label = page_title

=== modified file 'lib/lp/translations/browser/translations.py'
--- lib/lp/translations/browser/translations.py	2011-05-27 19:53:20 +0000
+++ lib/lp/translations/browser/translations.py	2011-07-01 11:27:27 +0000
@@ -56,13 +56,9 @@
             rootsite='answers')
 
 
-class RosettaApplicationView(TranslationsMixin):
+class RosettaApplicationView(LaunchpadView, TranslationsMixin):
     """View for various top-level Translations pages."""
 
-    def __init__(self, context, request):
-        self.context = context
-        self.request = request
-
     @property
     def ubuntu_translationseries(self):
         ubuntu = getUtility(ILaunchpadCelebrities).ubuntu

=== modified file 'lib/lp/translations/javascript/sourcepackage_sharing_details.js'
--- lib/lp/translations/javascript/sourcepackage_sharing_details.js	2011-04-22 21:33:30 +0000
+++ lib/lp/translations/javascript/sourcepackage_sharing_details.js	2011-07-27 18:41:24 +0000
@@ -83,21 +83,6 @@
 });
 
 namespace.LinkCheckItem = LinkCheckItem;
-namespace.convert_cache = function(lp_client, cache) {
-    var new_cache = {};
-    var value;
-    var key;
-    for (key in cache){
-        if (cache.hasOwnProperty(key)){
-            value = cache[key];
-            if (value !== null){
-                value = lp_client.wrap_resource(value.self_link, value);
-            }
-            new_cache[key] = value;
-        }
-    }
-    return new_cache;
-};
 
 
 namespace.autoimport_modes = {
@@ -174,14 +159,8 @@
 }
 
 
-function form_url(entry, view_name) {
-    entry_url = Y.lp.get_url_path(entry.get('web_link'));
-    return entry_url + '/' + view_name + '/++form++';
-}
-
-
 function update_form(overlay, entry, view_name) {
-    var url = form_url(entry, view_name);
+    var url = Y.lp.client.get_form_url(entry, view_name);
     overlay.loadFormContentAndRender(url);
     overlay.render();
 }
@@ -212,7 +191,7 @@
     }
     form_data_entries.push('field.actions.' + action + '=ignored');
     var data = form_data_entries.join('&');
-    var url = form_url(entry, view_name);
+    var url = Y.lp.client.get_form_url(entry, view_name);
     config.method = 'POST';
     config.data = data;
     Y.io(url, config);
@@ -240,6 +219,8 @@
 
 function IOHandler(controller, check, error_handler) {
     that = this;
+    this.check = check;
+    this.controller = controller;
     if (!Y.Lang.isValue(error_handler)){
         this.error_handler = new Y.lp.client.ErrorHandler();
     }
@@ -254,6 +235,23 @@
     };
 }
 
+IOHandler.prototype.show_success = function(){
+    this.check.set('pending', false);
+    this.controller.update();
+    this.controller.flash_check_green(this.check);
+};
+
+IOHandler.prototype.refresh_from_model = function(model){
+    this.controller.update_from_model(model);
+    this.show_success();
+};
+
+IOHandler.prototype.refresh_config = function(){
+    return this.chain_config(
+        Y.bind("load_model", this.controller),
+        Y.bind('refresh_from_model', this));
+};
+
 
 /**
  * Return an LP client config using error_handler.
@@ -294,6 +292,9 @@
 };
 
 
+namespace.IOHandler = IOHandler;
+
+
 /**
  * This class is the controller for updating the TranslationSharingConfig.
  * It handles updating the HTML and the DB model.
@@ -317,17 +318,24 @@
      * @param branch_summary {Object} An object containing api_url, css,
      * description, value, title
      */
-    configure: function(config, branch_picker_config, unlink_overlay,
+    configure: function(model, branch_picker_config, unlink_overlay,
                         import_overlay, usage_overlay) {
         this.set('branch_picker_config', branch_picker_config);
-        this.set('source_package', config.context);
         this.set('unlink_overlay', unlink_overlay);
         this.set('import_overlay', import_overlay);
         this.set('usage_overlay', usage_overlay);
-        this.replace_productseries(config.productseries);
-        this.replace_product(config.product);
-        this.set_branch(config.upstream_branch);
-        this.set_permissions(config);
+        this.update_from_model(model);
+    },
+    update_from_model: function(model) {
+        this.set('source_package', model.context);
+        this.replace_productseries(model.productseries);
+        this.replace_product(model.product);
+        this.set_branch(model.upstream_branch);
+        this.set_permissions(model);
+    },
+    load_model: function(config){
+        var source_package = this.get('source_package');
+        Y.lp.client.load_model(source_package, '+sharing-details', config);
     },
     set_permissions: function(permissions){
         var usage = this.get('tsconfig').get('translations_usage');
@@ -420,37 +428,10 @@
             var source_package = that.get('source_package');
             config.parameters = {
                 productseries: productseries_summary.api_uri};
-            source_package.named_post(
-                'setPackagingReturnSharingDetailPermissions', config);
-        }
-        function get_productseries(config, permissions) {
-            that.set_permissions(permissions);
-            lp_client.get(productseries_summary.api_uri, config);
-        }
-        function cache_productseries(config, productseries) {
-            that.replace_productseries(productseries);
-            var branch_link = productseries.get('branch_link');
-            if (branch_link === null){
-                config.on.success(null);
-            }
-            else {
-                lp_client.get(branch_link, config);
-            }
-        }
-        function cache_branch(config, branch) {
-            that.set_branch(branch);
-            project_link = that.get('productseries').get('project_link');
-            lp_client.get(project_link, config);
-        }
-        function set_usage(product) {
-            that.replace_product(product);
-            productseries_check.set('pending', false);
-            that.update();
-            that.flash_check_green(productseries_check);
+            source_package.named_post('setPackaging', config);
         }
         var io_handler = new IOHandler(this, productseries_check);
-        save_productseries(io_handler.chain_config(
-            get_productseries, cache_productseries, cache_branch, set_usage));
+        save_productseries(io_handler.refresh_config());
     },
     remove_productseries: function(productseries_summary) {
         var that = this;
@@ -465,12 +446,10 @@
             that.replace_productseries(null);
             that.replace_product(null);
             that.set_branch(null);
-            productseries_check.set('pending', false);
-            that.update();
-            that.flash_check_green(productseries_check);
+            io_handler.show_success();
         }
         var io_handler = new IOHandler(this, productseries_check);
-        delete_packaging(io_handler.chain_config(set_checks));
+        delete_packaging(io_handler.get_config(set_checks));
     },
     select_branch: function(branch_summary) {
         var that = this;
@@ -478,35 +457,14 @@
         var branch_check = that.get('tsconfig').get('branch');
         var productseries = that.get('productseries');
 
-        /* Here begin a series of methods which each represent a step in
-         * setting the branch.  They each take a config to use in an lp_client
-         * call, except the last one.  This allows them to be chained
-         * together.
-         *
-         * They take full advantage of their access to variables in the
-         * closure, such as "that" and "branch_summary".
-         */
         function save_branch(config) {
             branch_check.set('pending', true);
             that.update_check(branch_check);
             productseries.set('branch_link', branch_summary.api_uri);
             productseries.lp_save(config);
         }
-        function get_branch(config){
-            lp_client.get(branch_summary.api_uri, config);
-        }
-        function set_link(config, branch){
-            that.set_branch(branch);
-            lp_client.get(productseries.get('self_link'), config);
-        }
-        function finish(new_productseries){
-            that.replace_productseries(new_productseries);
-            branch_check.set('pending', false);
-            that.update();
-            that.flash_check_green(branch_check);
-        }
         var io_handler = new IOHandler(this, branch_check);
-        save_branch(io_handler.chain_config(get_branch, set_link, finish));
+        save_branch(io_handler.refresh_config());
     },
     /**
      * Update the display of all checklist items.
@@ -531,6 +489,9 @@
     visible_check_selector: function(check) {
         return this.check_selector(check, check.get('complete'));
     },
+    spinner_selector: function(check) {
+        return this.visible_check_selector(check) + '-spinner';
+    },
     picker_selector: function(check, complete) {
         return this.check_selector(check, complete) + '-picker a';
     },
@@ -562,8 +523,7 @@
         if (incomplete_picker !== null) {
             incomplete_picker.toggleClass('unseen', hide_picker);
         }
-        var selector = this.visible_check_selector(check) + '-spinner';
-        var spinner = Y.one(selector);
+        var spinner = Y.one(this.spinner_selector(check));
         if (Y.Lang.isValue(spinner)){
             spinner.toggleClass('unseen', !check.get('pending'));
         }
@@ -583,7 +543,7 @@
 namespace.prepare = function(cache) {
     var sharing_controller = new namespace.TranslationSharingController();
     var lp_client = new Y.lp.client.Launchpad();
-    cache = namespace.convert_cache(lp_client, cache);
+    cache = lp_client.wrap_resource(null, cache);
     var branch_picker_config = {
         picker_activator: '#branch-incomplete-picker a',
         header : 'Select translation branch',
@@ -632,9 +592,7 @@
         handler = new IOHandler(sharing_controller, autoimport_check);
         function update_controller() {
             sharing_controller.set_autoimport_mode(mode);
-            autoimport_check.set('pending', false);
-            sharing_controller.update();
-            sharing_controller.flash_check_green(autoimport_check);
+            handler.show_success();
         }
         autoimport_check.set('pending', true);
         sharing_controller.update_check(autoimport_check);
@@ -645,7 +603,7 @@
          * the value stored in productseries.
          */
         product_series.removeAttr('http_etag');
-        product_series.lp_save(handler.chain_config(update_controller));
+        product_series.lp_save(handler.get_config(update_controller));
     });
     var autoimport = sharing_controller.get('tsconfig').get('autoimport');
     sharing_controller.set_check_picker(autoimport, import_overlay);
@@ -653,33 +611,11 @@
     var usage_overlay = create_form_overlay(
         '<h2>Configure translations<h2>', function(form_data) {
         var product = sharing_controller.get('product');
-        function reload_entry(config, entry) {
-            lp_client.get(entry.get('self_link'), config);
-        }
-        function get_productseries(config) {
-            var productseries = sharing_controller.get('productseries');
-            reload_entry(config, productseries);
-        }
-        function replace_productseries(config, new_productseries) {
-            sharing_controller.replace_productseries(new_productseries);
-            config.on.success();
-        }
-        function get_product(config) {
-            reload_entry(config, product);
-        }
-        function replace_product(new_product) {
-            sharing_controller.replace_product(new_product);
-            usage.set('pending', false);
-            sharing_controller.update();
-            sharing_controller.flash_check_green(usage);
-        }
         var io_handler = new IOHandler(
             sharing_controller, usage, new Y.lp.client.FormErrorHandler());
         usage.set('pending', true);
         sharing_controller.update_check(usage);
-        var config = io_handler.chain_config(
-            get_productseries, replace_productseries, get_product,
-            replace_product);
+        var config = io_handler.refresh_config();
         submit_form(
             config, form_data, product, '+configure-translations', 'change');
     });
@@ -690,4 +626,5 @@
     sharing_controller.update();
 };
 }, "0.1", {"requires": [
-    'lp', 'lp.app.errors', 'lp.app.picker', 'oop', 'lp.client']});
+    'lp', 'lp.app.errors', 'lp.app.picker', 'oop', 'lp.client',
+    'lazr.anim']});

=== modified file 'lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.html'
--- lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.html	2011-07-08 06:06:15 +0000
+++ lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.html	2011-07-15 20:47:42 +0000
@@ -12,6 +12,7 @@
 
     <script type="text/javascript" src="../../../app/javascript/client.js"></script>
     <script type="text/javascript" src="../../../app/javascript/lp.js"></script>
+    <script type="text/javascript" src="../../../app/javascript/anim/anim.js"></script>
 
     <!-- The module under test -->
     <script type="text/javascript" src="../sourcepackage_sharing_details.js"></script>

=== modified file 'lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.js'
--- lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.js	2011-07-08 06:21:11 +0000
+++ lib/lp/translations/javascript/tests/test_sourcepackage_sharing_details.js	2011-07-27 18:41:24 +0000
@@ -7,6 +7,7 @@
 
     var suite = new Y.Test.Suite("sourcepackage_sharing_details Tests");
     var namespace = Y.lp.translations.sourcepackage_sharing_details;
+    var IOHandler = namespace.IOHandler;
     var TranslationSharingConfig = namespace.TranslationSharingConfig;
     var TranslationSharingController = namespace.TranslationSharingController;
     var CheckItem = (
@@ -101,12 +102,12 @@
         },
         test_configure_empty: function() {
             var ctrl = new TranslationSharingController();
-            var cache = {
+            var model = {
                 productseries: null,
                 product: null,
                 upstream_branch: null
             };
-            ctrl.configure(cache, {});
+            ctrl.configure(model, {});
         },
         test_configure: function() {
             var cache = {
@@ -133,7 +134,7 @@
             };
             var ctrl = new TranslationSharingController();
             var lp_client = new Y.lp.client.Launchpad();
-            cache = test_ns.convert_cache(lp_client, cache);
+            var model = lp_client.wrap_resource(null, cache);
             var import_overlay = {
                 loadFormContentAndRender: function() {},
                 render: function() {},
@@ -145,7 +146,7 @@
             unlink_overlay = import_overlay;
             usage_overlay = import_overlay;
             ctrl.configure(
-                cache, {}, unlink_overlay, import_overlay, usage_overlay);
+                model, {}, unlink_overlay, import_overlay, usage_overlay);
             var tsconfig = ctrl.get('tsconfig');
             Y.Assert.areEqual(
                 tsconfig.get('product_series').get('text'), 'title1');
@@ -157,22 +158,66 @@
             Y.Assert.isTrue(tsconfig.get('autoimport').get('complete'));
             Y.Assert.isTrue(
                 tsconfig.get('translations_usage').get('complete'));
-            Y.Assert.areSame(ctrl.get('source_package'), cache.context);
+            Y.Assert.areSame(ctrl.get('source_package'), model.context);
         },
-        test_convert_cache: function() {
+        test_update_from_model: function() {
+            var null_model = {
+                productseries: null,
+                product: null,
+                upstream_branch: null
+            };
+
             var cache = {
-                foo: {
-                    self_link: 'http://foo',
-                    resource_type_link: 'http://foo_type'
-                },
-                bar: null,
-                baz: true
+                product: {
+                    translations_usage: test_ns.usage.launchpad,
+                    resource_type_link: 'http://product'
+                },
+                productseries: {
+                    title: 'title1',
+                    web_link: 'http://web1',
+                    translations_autoimport_mode: (
+                        test_ns.autoimport_modes.import_translations),
+                    resource_type_link: 'productseries'
+                },
+                upstream_branch: {
+                    unique_name: 'title2',
+                    web_link: 'http://web2',
+                    resource_type_link: 'branch'
+                },
+                context: {
+                    resource_type_link: 'http://sourcepackage'
+                },
+                user_can_change_branch: true
             };
+            var ctrl = new TranslationSharingController();
             var lp_client = new Y.lp.client.Launchpad();
-            cache = test_ns.convert_cache(lp_client, cache);
-            Y.Assert.isNull(cache.bar);
-            Y.Assert.isTrue(cache.baz);
-            Y.Assert.areEqual('http://foo', cache.foo.get('self_link'));
+            var model = lp_client.wrap_resource(null, cache);
+            var import_overlay = {
+                loadFormContentAndRender: function() {},
+                render: function() {},
+                get: function(ignore) {
+                    return Y.Node.create(
+                        '<p><a href="http://fake">fake</a></p>');
+                    }
+            };
+            unlink_overlay = import_overlay;
+            usage_overlay = import_overlay;
+            ctrl.configure(
+                null_model, {}, unlink_overlay, import_overlay,
+                usage_overlay);
+            ctrl.update_from_model(model);
+            var tsconfig = ctrl.get('tsconfig');
+            Y.Assert.areEqual(
+                tsconfig.get('product_series').get('text'), 'title1');
+            Y.Assert.areEqual(
+                tsconfig.get('product_series').get('url'), 'http://web1');
+            Y.Assert.areEqual(
+                tsconfig.get('branch').get('text'), 'lp:title2');
+            Y.Assert.isTrue(tsconfig.get('branch').get('user_authorized'));
+            Y.Assert.isTrue(tsconfig.get('autoimport').get('complete'));
+            Y.Assert.isTrue(
+                tsconfig.get('translations_usage').get('complete'));
+            Y.Assert.areSame(ctrl.get('source_package'), model.context);
         },
         test_set_permissions: function(){
             var ctrl = new TranslationSharingController();
@@ -339,6 +384,21 @@
             Y.Assert.isTrue(check.get('complete'));
         }
     }));
+    suite.add(new Y.Test.Case({
+        name: 'setup',
+        test_show_success: function(){
+            var controller = new TranslationSharingController();
+            var check = controller.get('tsconfig').get('branch');
+            check.set('pending', true);
+            controller.update_check(check);
+            spinner = Y.one(controller.spinner_selector(check));
+            Y.Assert.isFalse(spinner.hasClass('unseen'));
+            io_handler = new IOHandler(controller, check);
+            io_handler.show_success();
+            Y.Assert.isFalse(check.get('pending'));
+            Y.Assert.isTrue(spinner.hasClass('unseen'));
+        }
+    }));
 
     Y.lp.testing.Runner.run(suite);
 

=== modified file 'versions.cfg'
--- versions.cfg	2011-07-27 05:48:20 +0000
+++ versions.cfg	2011-07-28 13:41:31 +0000
@@ -35,7 +35,7 @@
 lazr.delegates = 1.2.0
 lazr.enum = 1.1.3
 lazr.lifecycle = 1.1
-lazr.restful = 0.18.1
+lazr.restful = 0.19
 lazr.restfulclient = 0.12.0
 lazr.smtptest = 1.3
 lazr.testing = 0.1.1
@@ -130,7 +130,7 @@
 zc.lockfile = 1.0.0
 zc.recipe.egg = 1.3.2
 zc.zservertracelog = 1.1.5
-ZConfig = 2.7.1
+ZConfig = 2.9.1dev-20110728
 zdaemon = 2.0.4
 ZODB3 = 3.9.2
 zodbcode = 3.4.0

